


import pandas as pd

from strategies import Strategies
from utils import logger
from utils.global_vars import get_logger

pd.options.mode.chained_assignment = None  # default='warn'


class MACDCross(Strategies):
    def __init__(self, input_data: dict, fast_period=12, slow_period=26, signal_period=9, observation=100):
        self.MACD_FAST = fast_period
        self.MACD_SLOW = slow_period
        self.MACD_SIGNAL = signal_period
        self.OBSERVATION = observation
        self.default_logger = get_logger("macd_cross")

        super().__init__(input_data)
        self.parse_data()

    def parse_data(self, stock_list: list = None, latest_data: pd.DataFrame = None, backtesting: bool = False):
        # Received New Data => Parse it Now to input_data
        if latest_data is not None:
            # Only need to update MACD for the stock_code with new data
            stock_list = [latest_data['code'][0]]

            # Remove records with duplicate time_key. Always use the latest data to override
            time_key = latest_data['time_key'][0]
            self.input_data[stock_list[0]].drop(
                self.input_data[stock_list[0]][self.input_data[stock_list[0]].time_key == time_key].index,
                inplace=True)
            # Append empty columns and concat at the bottom
            latest_data = pd.concat([latest_data, pd.DataFrame(columns=['MACD', 'MACD_signal', 'MACD_hist'])])
            self.input_data[stock_list[0]] = pd.concat([self.input_data[stock_list[0]], latest_data])
        elif stock_list is not None:
            # Override Updated Stock List
            stock_list = stock_list
        else:
            stock_list = self.input_data.keys()

        # Calculate MACD for the stock_list
        for stock_code in stock_list:
            # Need to truncate to a maximum length for low-latency
            if not backtesting:
                self.input_data[stock_code] = self.input_data[stock_code].iloc[
                                              -min(self.OBSERVATION, self.input_data[stock_code].shape[0]):]
            self.input_data[stock_code][['open', 'close', 'high', 'low']] = self.input_data[stock_code][
                ['open', 'close', 'high', 'low']].apply(pd.to_numeric)

            # MACD = EMA-Fast - EMA-Slow. Signal = EMA(MACD, Smooth-period)
            ema_fast = self.input_data[stock_code]['close'].ewm(span=self.MACD_FAST, adjust=False).mean()
            ema_slow = self.input_data[stock_code]['close'].ewm(span=self.MACD_SLOW, adjust=False).mean()
            self.input_data[stock_code]['MACD'] = ema_fast - ema_slow
            self.input_data[stock_code]['MACD_signal'] = self.input_data[stock_code]['MACD'].ewm(span=self.MACD_SIGNAL,
                                                                                                 adjust=False).mean()
            # MACD_hist = (MACD - MACD_signal) * 2
            self.input_data[stock_code]['MACD_hist'] = (self.input_data[stock_code]['MACD'] -
                                                        self.input_data[stock_code]['MACD_signal']) * 2

            self.input_data[stock_code].reset_index(drop=True, inplace=True)

    def buy(self, stock_code) -> bool:
        # Crossover between MACD and Signal (Single Point Determined)
        current_record, previous_record = self.get_current_and_previous_record(stock_code)
        buy_decision = float(current_record['MACD']) > float(current_record['MACD_signal']) and float(
            previous_record['MACD']) <= float(previous_record['MACD_signal'])
        if buy_decision:
            self.default_logger.info(
                f"Buy Decision: {current_record['time_key']} based on \n {pd.concat([previous_record.to_frame().transpose(), current_record.to_frame().transpose()], axis=0)}")

        return buy_decision

    def sell(self, stock_code) -> bool:
        # Crossover between Signal and MACD (Single Point Determined)
        current_record, previous_record = self.get_current_and_previous_record(stock_code)
        sell_decision = float(current_record['MACD']) < float(current_record['MACD_signal']) and float(
            previous_record['MACD']) >= float(previous_record['MACD_signal'])
        if sell_decision:
            self.default_logger.info(
                f"Sell Decision: {current_record['time_key']} based on \n {pd.concat([previous_record.to_frame().transpose(), current_record.to_frame().transpose()], axis=0)}")

        return sell_decision



"""
AnalysisDataManager - 分析页面数据管理模块

负责分析页面的股票数据获取、历史数据处理、技术指标计算和实时数据更新
"""

import asyncio
import json
import os
from datetime import datetime, timedelta
from typing import Dict, List, Optional, Any, Tuple
from dataclasses import dataclass
from textual import work
from base.monitor import StockData, MarketStatus, ConnectionStatus
from base.futu_class import KLineData, OrderBookData, BrokerQueueData
from modules.futu_market import FutuMarket
from utils.global_vars import get_logger
from utils.global_vars import PATH_DATA

# 时间周期常量
TIME_PERIODS = {
    'D': 'K_DAY',      # 日线
    'W': 'K_WEEK',     # 周线  
    'M': 'K_MON'       # 月线
}

# 数据缓存配置
KLINE_CACHE_DAYS = 90      # K线数据缓存天数
KLINE_REFRESH_SEC = 60     # K线数据刷新间隔(秒) - 1分钟更新一次
ORDERBOOK_REFRESH_SEC = 3   # 五档数据刷新间隔(秒)
TICK_REFRESH_SEC = 1        # 逐笔数据刷新间隔(秒)
BASIC_INFO_REFRESH_SEC = 1  # 基础信息刷新间隔(秒)


@dataclass
class AnalysisDataSet:
    """分析页面数据集"""
    stock_code: str
    stock_name: str
    basic_info: Dict[str, Any]          # 基础信息
    realtime_quote: Dict[str, Any]      # 实时报价
    kline_data: List[KLineData]         # K线数据
    orderbook_data: Optional[OrderBookData]  # 五档数据
    tick_data: List[Dict[str, Any]]     # 逐笔数据
    broker_queue: Optional[BrokerQueueData]  # 经纪队列
    capital_flow: Dict[str, Any]        # 资金流向数据
    technical_indicators: Dict[str, Any] # 技术指标
    last_update: datetime


class AnalysisDataManager:
    """
    分析数据管理器
    负责分析页面的所有数据获取和处理
    """
    
    def __init__(self, app_core, futu_market: FutuMarket):
        """初始化分析数据管理器"""
        self.app_core = app_core
        self.futu_market = futu_market
        self.logger = get_logger(__name__)
        
        # 当前分析的股票（主要用于UI显示，可能有多个标签页打开不同股票）
        self.current_stock_code: Optional[str] = None
        self.current_time_period: str = 'D'  # 默认日线
        
        # 数据缓存
        self.analysis_data_cache: Dict[str, AnalysisDataSet] = {}
        
        # 按股票代码管理的实时更新任务
        self.stock_tasks: Dict[str, Dict[str, Optional[asyncio.Task]]] = {}  
        # 结构: {stock_code: {'realtime': task, 'orderbook': task, 'tick': task}}
        
        # 活跃股票集合（有标签页打开的股票）
        self.active_stocks: set = set()
        
        # 缓存上次的格式化数据值，用于检测变化并实现闪烁效果
        self.last_formatted_values: Dict[str, Dict[str, str]] = {}  # {stock_code: {data_type: formatted_value}}

        self.initialize_data_managers()
        self.logger.info("AnalysisDataManager 初始化完成")
    
    def initialize_data_managers(self) -> None:
        """初始化数据管理器"""
        try:
            connect_success = self.futu_market.check()
            if connect_success:
                self.logger.debug("富途API连接成功")
            else:
                self.logger.warning("富途API连接失败")
        except Exception as e:
            self.logger.error(f"富途API连接失败: {e}")

    def cleanup_futu_market(self) -> None:
        """清理富途市场连接"""
        try:
            if hasattr(self.futu_market, 'close'):
                self.futu_market.close()
                self.logger.info("富途市场连接已清理")
        except Exception as e:
            self.logger.warning(f"清理富途市场连接时出错: {e}")

    async def set_current_stock(self, stock_code: str) -> bool:
        """设置当前分析的股票并启动其实时更新任务"""
        try:
            self.current_stock_code = stock_code
            
            # 将股票加入活跃股票集合
            self.active_stocks.add(stock_code)
            
            self.logger.info(f"切换到股票分析: {stock_code}")
            
            # 加载股票分析数据
            await self.load_analysis_data(stock_code)
            
            # 启动该股票的实时更新任务（如果尚未启动）
            await self._start_stock_update_tasks(stock_code)
            
            return True
            
        except Exception as e:
            self.logger.error(f"设置当前分析股票失败: {e}")
            return False
    
    async def load_analysis_data(self, stock_code: str) -> Optional[AnalysisDataSet]:
        """加载股票的完整分析数据"""
        try:
            _stock_code = stock_code.replace("_", ".") if "_" in stock_code else stock_code
            self.logger.info(f"开始加载股票 {stock_code} 的分析数据")
            
            # 检查缓存
            if stock_code in self.analysis_data_cache:
                cache_data = self.analysis_data_cache[stock_code]
                # 如果缓存数据不超过1分钟，直接返回
                if (datetime.now() - cache_data.last_update).seconds < 60:
                    self.logger.debug(f"使用缓存的分析数据: {stock_code}")
                    return cache_data
            
            # 并行获取各种数据
            loop = asyncio.get_event_loop()
            
            # 1. 获取基础信息
            basic_info_task = loop.run_in_executor(
                None, self._get_stock_basic_info, _stock_code
            )
            
            # 2. 获取实时报价
            realtime_quote_task = loop.run_in_executor(
                None, self._get_realtime_quote, _stock_code
            )
            
            # 3. 获取K线数据
            kline_data_task = loop.run_in_executor(
                None, self._get_kline_data, _stock_code, self.current_time_period
            )
            
            # 4. 获取五档数据
            orderbook_data_task = loop.run_in_executor(
                None, self._get_orderbook_data, _stock_code
            )
            
            # 5. 获取逐笔数据
            tick_data_task = loop.run_in_executor(
                None, self._get_tick_data, _stock_code
            )
            
            # 6. 获取经纪队列数据
            broker_queue_task = loop.run_in_executor(
                None, self._get_broker_queue_data, _stock_code
            )
            
            # 等待所有任务完成
            results = await asyncio.gather(
                basic_info_task,
                realtime_quote_task,
                kline_data_task,
                orderbook_data_task,
                tick_data_task,
                broker_queue_task,
                return_exceptions=True
            )
            
            basic_info, realtime_quote, kline_data, orderbook_data, tick_data, broker_queue = results
            
            # 处理异常结果
            if isinstance(basic_info, Exception):
                self.logger.error(f"获取基础信息失败: {basic_info}")
                basic_info = {}
            if isinstance(realtime_quote, Exception):
                self.logger.error(f"获取实时报价失败: {realtime_quote}")
                realtime_quote = {}
            if isinstance(kline_data, Exception):
                self.logger.error(f"获取K线数据失败: {kline_data}")
                kline_data = []
            if isinstance(orderbook_data, Exception):
                self.logger.error(f"获取五档数据失败: {orderbook_data}")
                orderbook_data = None
            if isinstance(tick_data, Exception):
                self.logger.error(f"获取逐笔数据失败: {tick_data}")
                tick_data = []
            if isinstance(broker_queue, Exception):
                self.logger.error(f"获取经纪队列失败: {broker_queue}")
                broker_queue = None
            
            # 计算技术指标
            technical_indicators = await self._calculate_technical_indicators(kline_data)

            # 获取资金流向数据
            capital_flow = await self._get_capital_flow_data(_stock_code)

            # 创建分析数据集
            analysis_data = AnalysisDataSet(
                stock_code=stock_code,
                stock_name=basic_info.get('name', stock_code),
                basic_info=basic_info,
                realtime_quote=realtime_quote,
                kline_data=kline_data,
                orderbook_data=orderbook_data,
                tick_data=tick_data,
                broker_queue=broker_queue,
                capital_flow=capital_flow,
                technical_indicators=technical_indicators,
                last_update=datetime.now()
            )
            
            # 缓存数据
            self.analysis_data_cache[stock_code] = analysis_data
            
            self.logger.info(f"股票 {stock_code} 分析数据加载完成")
            return analysis_data
            
        except Exception as e:
            self.logger.error(f"加载分析数据失败: {e}")
            return None
    
    def _get_stock_basic_info(self, stock_code: str) -> Dict[str, Any]:
        """获取股票基础信息"""
        try:
            # 从app_core缓存获取基础信息，优先获取股票名称
            cached_info = self.app_core.stock_basicinfo_cache.get(stock_code, {})
            stock_name = cached_info.get('name', stock_code)
            
            # 使用get_market_snapshot获取市场快照数据
            snapshots = self.futu_market.get_market_snapshot([stock_code])
            self.logger.debug(f"获取股票 {stock_code} 的市场快照数据: {snapshots}")
            if snapshots and len(snapshots) > 0:
                snapshot = snapshots[0]
                
                # 从快照数据构建基础信息
                basic_info = {
                    'code': getattr(snapshot, 'code', stock_code),
                    'name': stock_name,  # 使用缓存的股票名称
                    'last_price': getattr(snapshot, 'last_price', 0.0),
                    'prev_close_price': getattr(snapshot, 'prev_close_price', 0.0),
                    'update_time': getattr(snapshot, 'update_time', ''),
                    'volume': getattr(snapshot, 'volume', 0),
                    'turnover': getattr(snapshot, 'turnover', 0.0),
                    'turnover_rate': getattr(snapshot, 'turnover_rate', 0.0),
                    'amplitude': getattr(snapshot, 'amplitude', 0.0)
                }
                
                # 如果有缓存的详细信息，添加到基础信息中
                if cached_info:
                    basic_info.update({
                        'lot_size': cached_info.get('lot_size', 0),
                        'stock_type': cached_info.get('stock_type', ''),
                        'listing_date': cached_info.get('listing_date', None),
                    })
                
                return basic_info
            
            # 如果获取快照失败，返回基础信息
            return {
                'code': stock_code, 
                'name': stock_name,
                'last_price': 0.0,
                'prev_close_price': 0.0,
                'update_time': '',
                'volume': 0,
                'turnover': 0.0,
                'turnover_rate': 0.0,
                'amplitude': 0.0
            }
            
        except Exception as e:
            self.logger.error(f"获取股票基础信息失败: {e}")
            return {
                'code': stock_code, 
                'name': stock_code,
                'last_price': 0.0,
                'prev_close_price': 0.0,
                'update_time': '',
                'volume': 0,
                'turnover': 0.0,
                'turnover_rate': 0.0,
                'amplitude': 0.0
            }
    
    def _get_realtime_quote(self, stock_code: str) -> Dict[str, Any]:
        """获取实时报价数据"""
        try:
            quotes = self.futu_market.get_stock_quote([stock_code])
            if quotes and len(quotes) > 0:
                quote = quotes[0]
                return {
                    'code': quote.code,
                    'cur_price': getattr(quote, 'cur_price', 0),
                    'prev_close_price': getattr(quote, 'prev_close_price', 0),
                    'open_price': getattr(quote, 'open_price', 0),
                    'high_price': getattr(quote, 'high_price', 0),
                    'low_price': getattr(quote, 'low_price', 0),
                    'volume': getattr(quote, 'volume', 0),
                    'turnover': getattr(quote, 'turnover', 0),
                    'change_rate': getattr(quote, 'change_rate', 0),
                    'change_val': getattr(quote, 'change_val', 0),
                    'amplitude': getattr(quote, 'amplitude', 0),
                    'turnover_rate': getattr(quote, 'turnover_rate', 0),
                }
            return {}
            
        except Exception as e:
            self.logger.error(f"获取实时报价失败: {e}")
            return {}

    def _get_kline_data(self, stock_code: str, period: str, num: int = 100) -> List[KLineData]:
        """获取K线数据"""
        try:
            # 直接从API获取K线数据，不使用缓存
            kline_type = TIME_PERIODS.get(period, 'K_DAY')
            self.logger.debug(f"获取K线数据: stock={stock_code}, period={kline_type}, num={num}")

            # 首先尝试获取当前K线数据
            kline_data = self.futu_market.get_cur_kline(
                [stock_code], num=num, ktype=kline_type
            )

            if kline_data and len(kline_data) > 0:
                self.logger.debug(f"通过get_cur_kline成功获取{len(kline_data)}条K线数据")
                return kline_data
            else:
                # get_cur_kline返回空数据，尝试使用历史K线数据作为回退
                self.logger.warning(f"get_cur_kline返回空数据，尝试使用历史K线数据: {stock_code}")
                return self._get_history_kline_fallback(stock_code, kline_type, num)

        except Exception as e:
            self.logger.error(f"获取K线数据失败: stock={stock_code}, period={period}, error={e}")
            # 异常情况下也尝试使用历史数据
            try:
                return self._get_history_kline_fallback(stock_code, TIME_PERIODS.get(period, 'K_DAY'), num)
            except Exception as fallback_e:
                self.logger.error(f"历史K线数据回退也失败: {fallback_e}")
                return []

    def _get_history_kline_fallback(self, stock_code: str, kline_type: str, num: int = 100) -> List[KLineData]:
        """使用历史K线数据作为回退方案"""
        try:
            from datetime import datetime, timedelta

            # 计算日期范围：从今天往前推num天
            end_date = datetime.now().strftime('%Y-%m-%d')
            start_date = (datetime.now() - timedelta(days=num + 30)).strftime('%Y-%m-%d')  # 多取30天确保有足够数据

            self.logger.debug(f"尝试获取历史K线数据: {stock_code}, {start_date} ~ {end_date}, type={kline_type}")

            # 调用富途API的历史K线接口
            history_klines = self.futu_market.client.quote.get_history_kline(
                stock_code, start_date, end_date, kline_type, autype="qfq"
            )

            if history_klines and len(history_klines) > 0:
                # 只取最近的num条数据
                recent_klines = history_klines[-num:] if len(history_klines) > num else history_klines
                self.logger.info(f"通过历史K线数据成功获取{len(recent_klines)}条数据（共{len(history_klines)}条）")
                return recent_klines
            else:
                self.logger.warning(f"历史K线数据也为空: {stock_code}")
                return []

        except Exception as e:
            self.logger.error(f"获取历史K线数据失败: {stock_code}, error={e}")
            return []

    def _get_orderbook_data(self, stock_code: str) -> Optional[OrderBookData]:
        """获取五档买卖盘数据"""
        try:
            orderbook = self.futu_market.get_order_book(stock_code)
            return orderbook
            
        except Exception as e:
            self.logger.error(f"获取五档数据失败: {e}")
            return None
    
    def _get_tick_data(self, stock_code: str, num: int = 50) -> List[Dict[str, Any]]:
        """获取逐笔交易数据"""
        try:
            tick_data = self.futu_market.get_rt_ticker(stock_code)
            if tick_data is not None and not tick_data.empty:
                # 转换为字典格式
                return tick_data.to_dict('records')
            return []
            
        except Exception as e:
            self.logger.error(f"获取逐笔数据失败: {e}")
            return []
    
    def _get_broker_queue_data(self, stock_code: str) -> Optional[BrokerQueueData]:
        """获取经纪队列数据"""
        try:
            broker_queue = self.futu_market.get_broker_queue(stock_code)
            return broker_queue

        except Exception as e:
            self.logger.error(f"获取经纪队列失败: {e}")
            return None

    async def _get_capital_flow_data(self, stock_code: str) -> Dict[str, Any]:
        """获取资金流向数据

        Args:
            stock_code: 股票代码

        Returns:
            Dict: 资金流向数据字典，包含主力、超大单、大单、中单、小单等
        """
        try:
            loop = asyncio.get_event_loop()
            capital_flow_list = await loop.run_in_executor(
                None, self.futu_market.get_capital_flow, stock_code, "INTRADAY"
            )

            if not capital_flow_list or len(capital_flow_list) == 0:
                self.logger.debug(f"股票 {stock_code} 无资金流向数据")
                return {}

            # 获取最新的资金流向数据（列表中的最后一条）
            latest_flow = capital_flow_list[-1]

            capital_flow = {
                'main_in_flow': getattr(latest_flow, 'main_in_flow', 0),
                'super_in_flow': getattr(latest_flow, 'super_in_flow', 0),
                'big_in_flow': getattr(latest_flow, 'big_in_flow', 0),
                'mid_in_flow': getattr(latest_flow, 'mid_in_flow', 0),
                'sml_in_flow': getattr(latest_flow, 'sml_in_flow', 0),
                'capital_flow_item_time': getattr(latest_flow, 'capital_flow_item_time', ''),
                'last_valid_time': getattr(latest_flow, 'last_valid_time', '')
            }

            self.logger.info(f"✓ 成功获取股票 {stock_code} 资金流向: 主力净流入={capital_flow['main_in_flow']:.2f}")
            return capital_flow

        except Exception as e:
            self.logger.error(f"获取资金流向数据失败: {e}")
            return {}

    async def _calculate_technical_indicators(self, kline_data: List[KLineData]) -> Dict[str, Any]:
        """计算技术指标"""
        try:
            if not kline_data or len(kline_data) < 20:
                return {}
            
            # 提取价格数据
            closes = [k.close for k in kline_data]
            highs = [k.high for k in kline_data]
            lows = [k.low for k in kline_data]
            volumes = [k.volume for k in kline_data]
            
            # 计算移动平均线
            ma5 = self._calculate_ma(closes, 5)
            ma10 = self._calculate_ma(closes, 10)
            ma20 = self._calculate_ma(closes, 20)
            ma60 = self._calculate_ma(closes, 60)
            
            # 计算RSI
            rsi = self._calculate_rsi(closes, 14)
            
            # 计算MACD
            macd_data = self._calculate_macd(closes)
            
            return {
                'ma5': ma5[-1] if ma5 else 0,
                'ma10': ma10[-1] if ma10 else 0,
                'ma20': ma20[-1] if ma20 else 0,
                'ma60': ma60[-1] if ma60 else 0,
                'rsi': rsi[-1] if rsi else 0,
                'macd': macd_data,
                'price_trend': self._analyze_price_trend(closes, ma20),
                'volume_trend': self._analyze_volume_trend(volumes),
            }
            
        except Exception as e:
            self.logger.error(f"计算技术指标失败: {e}")
            return {}
    
    def _calculate_ma(self, prices: List[float], period: int) -> List[float]:
        """计算移动平均线"""
        if len(prices) < period:
            return []
        
        ma_values = []
        for i in range(period - 1, len(prices)):
            ma = sum(prices[i - period + 1:i + 1]) / period
            ma_values.append(ma)
        
        return ma_values
    
    def _calculate_rsi(self, prices: List[float], period: int = 14) -> List[float]:
        """计算RSI指标"""
        if len(prices) < period + 1:
            return []
        
        gains = []
        losses = []
        
        for i in range(1, len(prices)):
            change = prices[i] - prices[i - 1]
            if change > 0:
                gains.append(change)
                losses.append(0)
            else:
                gains.append(0)
                losses.append(abs(change))
        
        rsi_values = []
        for i in range(period - 1, len(gains)):
            avg_gain = sum(gains[i - period + 1:i + 1]) / period
            avg_loss = sum(losses[i - period + 1:i + 1]) / period
            
            if avg_loss == 0:
                rsi = 100
            else:
                rs = avg_gain / avg_loss
                rsi = 100 - (100 / (1 + rs))
            
            rsi_values.append(rsi)
        
        return rsi_values
    
    def _calculate_macd(self, prices: List[float]) -> Dict[str, Any]:
        """计算MACD指标"""
        if len(prices) < 26:
            return {'dif': 0, 'dea': 0, 'histogram': 0}
        
        # 计算EMA
        ema12 = self._calculate_ema(prices, 12)
        ema26 = self._calculate_ema(prices, 26)
        
        if not ema12 or not ema26:
            return {'dif': 0, 'dea': 0, 'histogram': 0}
        
        # 计算DIF
        dif = [ema12[i] - ema26[i] for i in range(len(ema26))]
        
        # 计算DEA (DIF的9日EMA)
        dea = self._calculate_ema(dif, 9)
        
        if not dea:
            return {'dif': dif[-1] if dif else 0, 'dea': 0, 'histogram': 0}
        
        # 计算MACD柱状图
        histogram = dif[-1] - dea[-1] if dea else 0
        
        return {
            'dif': dif[-1] if dif else 0,
            'dea': dea[-1] if dea else 0,
            'histogram': histogram
        }
    
    def _calculate_ema(self, prices: List[float], period: int) -> List[float]:
        """计算指数移动平均线"""
        if len(prices) < period:
            return []
        
        multiplier = 2 / (period + 1)
        ema_values = [prices[0]]  # 第一个EMA值等于第一个价格
        
        for i in range(1, len(prices)):
            ema = (prices[i] * multiplier) + (ema_values[-1] * (1 - multiplier))
            ema_values.append(ema)
        
        return ema_values
    
    def _analyze_price_trend(self, prices: List[float], ma20: List[float]) -> str:
        """分析价格趋势"""
        if not prices or not ma20 or len(prices) < 5:
            return "无明确趋势"
        
        current_price = prices[-1]
        current_ma20 = ma20[-1] if ma20 else current_price
        
        # 价格与均线比较
        if current_price > current_ma20 * 1.02:
            return "强势上涨"
        elif current_price > current_ma20:
            return "温和上涨"
        elif current_price < current_ma20 * 0.98:
            return "弱势下跌"
        else:
            return "震荡整理"
    
    def _analyze_volume_trend(self, volumes: List[int]) -> str:
        """分析成交量趋势"""
        if not volumes or len(volumes) < 5:
            return "成交量不足"
        
        recent_avg = sum(volumes[-5:]) / 5
        historical_avg = sum(volumes[:-5]) / max(1, len(volumes) - 5)
        
        if recent_avg > historical_avg * 1.5:
            return "放量"
        elif recent_avg < historical_avg * 0.7:
            return "缩量"
        else:
            return "量能平稳"
    
    async def change_time_period(self, period: str) -> bool:
        """切换时间周期"""
        try:
            if period not in TIME_PERIODS:
                self.logger.error(f"不支持的时间周期: {period}")
                return False
            
            if period == self.current_time_period:
                return True
            
            self.current_time_period = period
            self.logger.info(f"切换到时间周期: {period}")
            
            # 重新加载当前股票的K线数据
            if self.current_stock_code:
                await self.load_analysis_data(self.current_stock_code)
            
            return True
            
        except Exception as e:
            self.logger.error(f"切换时间周期失败: {e}")
            return False
    
    async def _start_stock_update_tasks(self, stock_code: str):
        """启动指定股票的实时更新任务（仅在交易时间内启动）"""
        try:
            # 如果该股票的任务已经在运行，跳过
            if stock_code in self.stock_tasks:
                running_tasks = [task for task in self.stock_tasks[stock_code].values() 
                               if task and not task.done()]
                if running_tasks:
                    self.logger.info(f"股票 {stock_code} 的实时更新任务已在运行")
                    return
            
            # 检查市场状态，只有在交易时间内才启动实时更新任务
            is_trading_time = await self._check_market_trading_status(stock_code)
            if not is_trading_time:
                self.logger.info(f"股票 {stock_code} 不在交易时间内，跳过启动实时更新任务")
                return
            
            # 初始化该股票的任务字典
            if stock_code not in self.stock_tasks:
                self.stock_tasks[stock_code] = {'realtime': None, 'orderbook': None, 'tick': None, 'basic_info': None, 'kline': None}
            
            # 启动K线数据更新任务
            self.stock_tasks[stock_code]['kline'] = asyncio.create_task(
                self._kline_update_loop(stock_code)
            )
            
            # 启动五档数据更新任务
            self.stock_tasks[stock_code]['orderbook'] = asyncio.create_task(
                self._orderbook_update_loop(stock_code)
            )
            
            # 启动逐笔数据更新任务
            self.stock_tasks[stock_code]['tick'] = asyncio.create_task(
                self._tick_update_loop(stock_code)
            )
            
            # 启动基础信息数据更新任务
            self.stock_tasks[stock_code]['basic_info'] = asyncio.create_task(
                self._basic_info_update_loop(stock_code)
            )
            
            self.logger.info(f"股票 {stock_code} 的实时更新任务启动（交易时间内）")
            
        except Exception as e:
            self.logger.error(f"启动股票 {stock_code} 更新任务失败: {e}")
    
    async def _stop_stock_update_tasks(self, stock_code: str):
        """停止指定股票的实时更新任务"""
        try:
            if stock_code not in self.stock_tasks:
                return
            
            tasks = list(self.stock_tasks[stock_code].values())
            
            for task in tasks:
                if task and not task.done():
                    task.cancel()
                    try:
                        await task
                    except asyncio.CancelledError:
                        pass
            
            # 清空该股票的任务
            del self.stock_tasks[stock_code]
            
            self.logger.info(f"股票 {stock_code} 的实时更新任务停止")
        
        except Exception as e:
            self.logger.error(f"停止股票 {stock_code} 更新任务失败: {e}")
    
    async def _stop_update_tasks(self):
        """停止所有实时更新任务"""
        try:
            for stock_code in list(self.stock_tasks.keys()):
                await self._stop_stock_update_tasks(stock_code)
            
            self.logger.info("所有分析页面更新任务已停止")
            
        except Exception as e:
            self.logger.error(f"停止更新任务失败: {e}")
    
    async def _orderbook_update_loop(self, stock_code: str):
        """五档数据更新循环"""
        while True:
            try:
                # 检查股票是否仍在活跃集合中
                if stock_code not in self.active_stocks:
                    self.logger.info(f"股票 {stock_code} 已不再活跃，停止五档数据更新")
                    break
                    
                loop = asyncio.get_event_loop()
                orderbook_data = await loop.run_in_executor(
                    None, self._get_orderbook_data, stock_code
                )
                    
                # 更新缓存
                if stock_code in self.analysis_data_cache:
                    self.analysis_data_cache[stock_code].orderbook_data = orderbook_data
                    self.analysis_data_cache[stock_code].last_update = datetime.now()
                
                await asyncio.sleep(ORDERBOOK_REFRESH_SEC)
                
            except asyncio.CancelledError:
                break
            except Exception as e:
                self.logger.error(f"五档数据更新错误: {e}")
                await asyncio.sleep(ORDERBOOK_REFRESH_SEC)
    
    async def _tick_update_loop(self, stock_code: str):
        """逐笔数据更新循环"""
        while True:
            try:
                # 检查股票是否仍在活跃集合中
                if stock_code not in self.active_stocks:
                    self.logger.info(f"股票 {stock_code} 已不再活跃，停止逐笔数据更新")
                    break
                    
                loop = asyncio.get_event_loop()
                tick_data = await loop.run_in_executor(
                    None, self._get_tick_data, stock_code, 20
                )
                    
                # 更新缓存
                if stock_code in self.analysis_data_cache:
                    self.analysis_data_cache[stock_code].tick_data = tick_data
                    self.analysis_data_cache[stock_code].last_update = datetime.now()
                
                await asyncio.sleep(TICK_REFRESH_SEC)
                
            except asyncio.CancelledError:
                break
            except Exception as e:
                self.logger.error(f"逐笔数据更新错误: {e}")
                await asyncio.sleep(TICK_REFRESH_SEC)

    async def _basic_info_update_loop(self, stock_code: str):
        """基础信息数据更新循环"""
        while True:
            try:
                # 检查股票是否仍在活跃集合中
                if stock_code not in self.active_stocks:
                    self.logger.info(f"股票 {stock_code} 已不再活跃，停止基础信息数据更新")
                    break
                    
                loop = asyncio.get_event_loop()
                basic_info = await loop.run_in_executor(
                    None, self._get_stock_basic_info, stock_code
                )
                    
                # 更新缓存
                if stock_code in self.analysis_data_cache:
                    self.analysis_data_cache[stock_code].basic_info = basic_info
                    self.analysis_data_cache[stock_code].last_update = datetime.now()
                
                await asyncio.sleep(BASIC_INFO_REFRESH_SEC)
                
            except asyncio.CancelledError:
                break
            except Exception as e:
                self.logger.error(f"基础信息数据更新错误: {e}")
                await asyncio.sleep(BASIC_INFO_REFRESH_SEC)

    async def _kline_update_loop(self, stock_code: str):
        """K线数据更新循环"""
        while True:
            try:
                # 检查股票是否仍在活跃集合中
                if stock_code not in self.active_stocks:
                    self.logger.info(f"股票 {stock_code} 已不再活跃，停止K线数据更新")
                    break
                    
                loop = asyncio.get_event_loop()
                kline_data = await loop.run_in_executor(
                    None, self._get_kline_data, stock_code, self.current_time_period, 100
                )
                    
                # 更新缓存和重新计算技术指标
                if stock_code in self.analysis_data_cache:
                    self.analysis_data_cache[stock_code].kline_data = kline_data
                    
                    # 重新计算技术指标
                    technical_indicators = await self._calculate_technical_indicators(kline_data)
                    self.analysis_data_cache[stock_code].technical_indicators = technical_indicators
                    
                    self.analysis_data_cache[stock_code].last_update = datetime.now()
                    self.logger.debug(f"K线数据已更新: {stock_code}, 数据量: {len(kline_data)}")
                
                await asyncio.sleep(KLINE_REFRESH_SEC)
                
            except asyncio.CancelledError:
                break
            except Exception as e:
                self.logger.error(f"K线数据更新错误: {e}")
                await asyncio.sleep(KLINE_REFRESH_SEC)
    
    def get_current_analysis_data(self) -> Optional[AnalysisDataSet]:
        """获取当前分析数据"""
        if self.current_stock_code and self.current_stock_code in self.analysis_data_cache:
            return self.analysis_data_cache[self.current_stock_code]
        return None
    
    async def cleanup_stock_data(self, stock_code: str):
        """清理指定股票的分析数据和停止其实时更新任务"""
        try:
            # 从活跃股票集合中移除
            self.active_stocks.discard(stock_code)
            
            # 停止该股票的实时更新任务
            await self._stop_stock_update_tasks(stock_code)
            
            # 如果是当前分析的股票，清空当前股票标记
            if self.current_stock_code == stock_code:
                self.current_stock_code = None
                self.logger.info(f"已清空当前分析股票标记: {stock_code}")
            
            # 从缓存中删除该股票的数据
            if stock_code in self.analysis_data_cache:
                del self.analysis_data_cache[stock_code]
                self.logger.debug(f"已从缓存中删除股票 {stock_code} 的分析数据")
            
            
            self.logger.info(f"股票 {stock_code} 的分析数据和任务清理完成")
            
        except Exception as e:
            self.logger.error(f"清理股票 {stock_code} 分析数据失败: {e}")
    
    async def cleanup(self):
        """清理分析数据管理器"""
        try:
            await self._stop_update_tasks()
            self.analysis_data_cache.clear()
            self.last_formatted_values.clear()
            self.current_stock_code = None
            self.logger.info("AnalysisDataManager 清理完成")
            
        except Exception as e:
            self.logger.error(f"AnalysisDataManager 清理失败: {e}")
    
    # ==================== 数据格式化方法 ====================
    
    def format_basic_info(self, analysis_data=None) -> str:
        """格式化基础信息显示文本"""
        if not analysis_data:
            return "等待股票数据加载..."
            
        basic_info = analysis_data.basic_info
        realtime_quote = analysis_data.realtime_quote
        
        # 提取基础信息
        stock_code = basic_info.get('code', '未知')
        stock_name = basic_info.get('name', '未知')
        last_price = basic_info.get('last_price', '未知')
        prev_close_price = basic_info.get('prev_close_price', '未知')
        volume = basic_info.get('volume', '未知')
        turnover = basic_info.get('turnover', '未知')
        turnover_rate = basic_info.get('turnover_rate', '未知')
        amplitude = basic_info.get('amplitude', '未知')
        listing_date = basic_info.get('listing_date', '未知')

        # 提取实时数据用于计算市值等
        current_price = realtime_quote.get('cur_price', 0)
        volume = realtime_quote.get('volume', 0)
        
        # 判断市场
        market_map = {
            'HK': '港交所',
            'US': '纳斯达克/纽交所', 
            'SH': '上海证券交易所',
            'SZ': '深圳证券交易所'
        }
        market = stock_code.split('.')[0] if '.' in stock_code else 'Unknown'
        market_name = market_map.get(market, '未知市场')
        
        # 格式化显示文本
        info_text = (
            f"股票代码: {stock_code}    "
            f"名称: {stock_name}    "
            f"最新价格: {last_price}    "
            f"昨收盘价格: {prev_close_price}    "
            f"成交金额: {turnover}    "
            f"换手率: {turnover_rate}   "
            f"振幅: {amplitude}    "
        )
        
        if current_price > 0:
            market_cap = current_price * volume if volume > 0 else 0
            if market_cap > 100000000:  # 大于1亿
                market_cap_text = f"{market_cap/100000000:.1f}亿"
            else:
                market_cap_text = f"{market_cap/10000:.1f}万" if market_cap > 10000 else f"{market_cap:.0f}"
            info_text += f"当前价: {current_price:.2f}    市值估算: {market_cap_text}    "
            
        if listing_date and listing_date != '未知':
            info_text += f"上市日期: {listing_date}    "
            
        info_text += f"更新时间: {analysis_data.last_update.strftime('%Y-%m-%d %H:%M:%S')}"
        
        return info_text
    
    def format_realtime_quote(self, analysis_data=None) -> str:
        """格式化实时报价信息"""
        if not analysis_data or not analysis_data.realtime_quote:
            return "等待实时报价数据..."
        
        quote = analysis_data.realtime_quote
        
        # 提取报价数据
        cur_price = quote.get('cur_price', 0)
        prev_close = quote.get('prev_close_price', 0)
        open_price = quote.get('open_price', 0)
        high_price = quote.get('high_price', 0)
        low_price = quote.get('low_price', 0)
        volume = quote.get('volume', 0)
        turnover = quote.get('turnover', 0)
        change_rate = quote.get('change_rate', 0)
        change_val = quote.get('change_val', 0)
        amplitude = quote.get('amplitude', 0)
        turnover_rate = quote.get('turnover_rate', 0)
        
        # 格式化涨跌显示
        change_color = "green" if change_val >= 0 else "red"
        change_symbol = "↑" if change_val >= 0 else "↓"
        
        # 格式化成交量显示
        if volume > 100000000:
            volume_str = f"{volume/100000000:.1f}亿手"
        elif volume > 10000:
            volume_str = f"{volume/10000:.1f}万手"
        else:
            volume_str = f"{volume}手"
        
        # 格式化成交额显示
        if turnover > 100000000:
            turnover_str = f"{turnover/100000000:.1f}亿"
        elif turnover > 10000:
            turnover_str = f"{turnover/10000:.1f}万"
        else:
            turnover_str = f"{turnover:.0f}"
        
        quote_text = (
            f"最新价: [{change_color}]{cur_price:.2f}[/{change_color}] {change_symbol}    "
            f"涨跌幅: [{change_color}]{change_rate:+.2f}%[/{change_color}]    "
            f"涨跌额: [{change_color}]{change_val:+.2f}[/{change_color}]    "
            f"开盘: {open_price:.2f}    "
            f"最高: {high_price:.2f}    "
            f"最低: {low_price:.2f}    "
            f"成交量: {volume_str}    "
            f"成交额: {turnover_str}    "
            f"换手率: {turnover_rate:.2f}%    "
            f"振幅: {amplitude:.2f}%"
        )
        
        return quote_text
    
    def format_orderbook_data(self, analysis_data=None) -> str:
        """格式化五档买卖盘数据"""
        if not analysis_data or not analysis_data.orderbook_data:
            return "等待五档数据..."
        
        orderbook = analysis_data.orderbook_data
        
        # 构建五档显示（简化为三档）
        orderbook_text = ""
        
        # 卖盘买盘（从上到下）
        if hasattr(orderbook, 'ask_price_1') and orderbook.ask_price_1 > 0:
            orderbook_text += f"[bold red]卖一: {orderbook.ask_price_1:.2f}  {orderbook.ask_volume_1}手[/bold red]    "
        if hasattr(orderbook, 'bid_price_1') and orderbook.bid_price_1 > 0:
            orderbook_text += f"[bold green]买一: {orderbook.bid_price_1:.2f}  {orderbook.bid_volume_1}手[/bold green]\n"

        if hasattr(orderbook, 'ask_price_2') and orderbook.ask_price_2 > 0:
            orderbook_text += f"[bold red]卖二: {orderbook.ask_price_2:.2f}  {orderbook.ask_volume_2}手[/bold red]    "
        if hasattr(orderbook, 'bid_price_2') and orderbook.bid_price_2 > 0:
            orderbook_text += f"[bold green]买二: {orderbook.bid_price_2:.2f}  {orderbook.bid_volume_2}手[/bold green]\n"

        if hasattr(orderbook, 'ask_price_3') and orderbook.ask_price_3 > 0:
            orderbook_text += f"[bold red]卖三: {orderbook.ask_price_3:.2f}  {orderbook.ask_volume_3}手[/bold red]    "
        if hasattr(orderbook, 'bid_price_3') and orderbook.bid_price_3 > 0:
            orderbook_text += f"[bold green]买三: {orderbook.bid_price_3:.2f}  {orderbook.bid_volume_3}手[/bold green]\n"

        if hasattr(orderbook, 'ask_price_4') and orderbook.ask_price_4 > 0:
            orderbook_text += f"[bold red]卖四: {orderbook.ask_price_4:.2f}  {orderbook.ask_volume_4}手[/bold red]    "
        if hasattr(orderbook, 'bid_price_4') and orderbook.bid_price_4 > 0:
            orderbook_text += f"[bold green]买四: {orderbook.bid_price_4:.2f}  {orderbook.bid_volume_4}手[/bold green]\n"

        if hasattr(orderbook, 'ask_price_5') and orderbook.ask_price_5 > 0:
            orderbook_text += f"[bold red]卖五: {orderbook.ask_price_5:.2f}  {orderbook.ask_volume_5}手[/bold red]    "
        if hasattr(orderbook, 'bid_price_5') and orderbook.bid_price_5 > 0:
            orderbook_text += f"[bold green]买五: {orderbook.bid_price_5:.2f}  {orderbook.bid_volume_5}手[/bold green]\n"

        #orderbook_text += "──────────────────\n"
        
        
        # 计算委比和委差
        total_bid_vol = (getattr(orderbook, 'bid_volume_1', 0) + 
                        getattr(orderbook, 'bid_volume_2', 0) + 
                        getattr(orderbook, 'bid_volume_3', 0) +
                        getattr(orderbook, 'bid_volume_4', 0) +
                        getattr(orderbook, 'bid_volume_5', 0) +
                        getattr(orderbook, 'bid_volume_6', 0) +
                        getattr(orderbook, 'bid_volume_7', 0) +
                        getattr(orderbook, 'bid_volume_8', 0) +
                        getattr(orderbook, 'bid_volume_9', 0) +
                        getattr(orderbook, 'bid_volume_10', 0)
                        )
        total_ask_vol = (getattr(orderbook, 'ask_volume_1', 0) + 
                        getattr(orderbook, 'ask_volume_2', 0) + 
                        getattr(orderbook, 'ask_volume_3', 0) +
                        getattr(orderbook, 'ask_volume_4', 0) +
                        getattr(orderbook, 'ask_volume_5', 0) +
                        getattr(orderbook, 'ask_volume_6', 0) +
                        getattr(orderbook, 'ask_volume_7', 0) +
                        getattr(orderbook, 'ask_volume_8', 0) +
                        getattr(orderbook, 'ask_volume_9', 0) +
                        getattr(orderbook, 'ask_volume_10', 0)
                        ) 
        
        if (total_bid_vol + total_ask_vol) > 0:
            wei_bi = ((total_bid_vol - total_ask_vol) / (total_bid_vol + total_ask_vol)) * 100
            wei_cha = total_bid_vol - total_ask_vol
            
            if wei_cha > 10000:
                wei_cha_str = f"{wei_cha/10000:.1f}万手"
            else:
                wei_cha_str = f"{wei_cha}手"
            
            orderbook_text += f"📈 委比: {wei_bi:+.1f}%    "
            orderbook_text += f"📊 委差: {wei_cha_str}"
        
        return orderbook_text
    
    def format_tick_data(self, analysis_data=None) -> str:
        """格式化逐笔交易数据"""
        if not analysis_data or not analysis_data.tick_data:
            return "等待逐笔数据..."
        
        tick_data = analysis_data.tick_data
        
        if not tick_data or len(tick_data) == 0:
            return "暂无逐笔数据"
        
        tick_text = "[bold yellow]逐笔数据[/bold yellow]\n"
        
        # 显示最新的4-5笔交易
        recent_ticks = tick_data[:5] if len(tick_data) >= 5 else tick_data
        
        for tick in recent_ticks:
            time_str = tick.get('time', '')[:8]  # 只显示时分秒
            price = tick.get('price', 0)
            volume = tick.get('volume', 0)
            direction = tick.get('ticker_direction', '')
            
            # 根据买卖方向显示箭头和颜色
            if direction == 'BUY':
                direction_symbol = "[green]↑[/green]"
            elif direction == 'SELL':
                direction_symbol = "[red]↓[/red]"
            else:
                direction_symbol = "─"
            
            tick_text += f"{time_str}  {price:.2f}{direction_symbol}  {volume}手\n"
        
        return tick_text.rstrip('\n')
    
    def format_broker_queue(self, analysis_data=None) -> str:
        """格式化经纪队列数据"""
        if not analysis_data or not analysis_data.broker_queue:
            return "等待经纪队列数据..."
        
        broker = analysis_data.broker_queue
        
        # 检查是否为字典格式且包含必要的字段
        if not isinstance(broker, dict) or 'bid_frame_table' not in broker or 'ask_frame_table' not in broker:
            return "经纪队列数据格式异常"
        
        broker_text = "[bold cyan]经纪队列[/bold cyan]\n"
        
        # 处理买方队列
        bid_table = broker.get('bid_frame_table', {})
        if bid_table and 'bid_broker_pos' in bid_table and 'bid_broker_name' in bid_table:
            broker_text += "[green]买方队列:[/green]\n"
            bid_positions = self._group_brokers_by_position(
                bid_table.get('bid_broker_pos', {}),
                bid_table.get('bid_broker_name', {})
            )
            for pos in sorted(bid_positions.keys()):
                brokers = ', '.join(bid_positions[pos][:3])  # 最多显示3个经纪商
                broker_text += f"  {pos}档: {brokers}\n"
        else:
            broker_text += "[green]买方队列:[/green] 暂无数据\n"
        
        # 处理卖方队列
        ask_table = broker.get('ask_frame_table', {})
        if ask_table and 'ask_broker_pos' in ask_table and 'ask_broker_name' in ask_table:
            broker_text += "[red]卖方队列:[/red]\n"
            ask_positions = self._group_brokers_by_position(
                ask_table.get('ask_broker_pos', {}),
                ask_table.get('ask_broker_name', {})
            )
            for pos in sorted(ask_positions.keys()):
                brokers = ', '.join(ask_positions[pos][:3])  # 最多显示3个经纪商
                broker_text += f"  {pos}档: {brokers}"
                if pos < max(ask_positions.keys()):
                    broker_text += "\n"
        else:
            broker_text += "[red]卖方队列:[/red] 暂无数据"
        
        return broker_text
    
    def _group_brokers_by_position(self, positions, names):
        """按档位分组经纪商"""
        grouped = {}
        for idx, pos in positions.items():
            if pos not in grouped:
                grouped[pos] = set()
            if idx in names:
                broker_name = names[idx]
                # 简化经纪商名称显示
                simplified_name = self._simplify_broker_name(broker_name)
                grouped[pos].add(simplified_name)
        
        # 转换为列表并去重
        for pos in grouped:
            grouped[pos] = list(grouped[pos])
        
        return grouped
    
    def _simplify_broker_name(self, full_name):
        """简化经纪商名称以便显示"""
        # 移除常见后缀
        name = full_name.replace('有限公司', '').replace('(香港)', '').replace('证券', '')
        # 如果名称太长，截取前8个字符
        if len(name) > 8:
            name = name[:8] + '...'
        return name
    
    async def format_capital_flow(self, analysis_data=None) -> str:
        """格式化资金流向数据"""
        if not analysis_data or not analysis_data.capital_flow:
            # 如果没有资金流向数据，调用真实API获取资金流向信息
            return await self._generate_estimated_capital_flow(analysis_data)
        
        capital = analysis_data.capital_flow
        
        # 这里应该根据实际的资金流向数据结构进行格式化
        # 由于当前capital_flow为空字典，我们调用真实API获取数据
        return await self._generate_estimated_capital_flow(analysis_data)
    
    async def _generate_estimated_capital_flow(self, analysis_data=None) -> str:
        """调用get_capital_flow API获取真实的资金流向信息"""
        if not analysis_data or not analysis_data.realtime_quote:
            return "等待资金流向数据..."
        
        stock_code = self.current_stock_code
        if not stock_code:
            return "等待股票代码..."
        
        try:
            # 调用真实的资金流向API
            loop = asyncio.get_event_loop()
            capital_flow_list = await loop.run_in_executor(
                None, self.futu_market.get_capital_flow, stock_code, "INTRADAY"
            )
            
            if not capital_flow_list:
                # 如果API返回空数据，则回退到基于技术指标的估算
                return self._fallback_estimated_capital_flow(analysis_data)
            
            # 获取最新的资金流向数据（列表中的最后一条）
            latest_flow = capital_flow_list[-1]
            
            # 格式化资金流向数据
            main_flow = latest_flow.main_in_flow
            super_flow = latest_flow.super_in_flow
            big_flow = latest_flow.big_in_flow
            mid_flow = latest_flow.mid_in_flow
            sml_flow = latest_flow.sml_in_flow
            
            # 判断流入流出方向
            main_flow_direction = "流入" if main_flow > 0 else "流出"
            main_flow_color = "green" if main_flow > 0 else "red"
            
            # 格式化数值显示
            def format_capital(value):
                abs_value = abs(value)
                if abs_value > 100000000:
                    return f"{value/100000000:.1f}亿"
                elif abs_value > 10000:
                    return f"{value/10000:.1f}万"
                else:
                    return f"{value:.0f}"
            
            main_flow_str = format_capital(main_flow)
            super_flow_str = format_capital(super_flow)
            big_flow_str = format_capital(big_flow)
            mid_flow_str = format_capital(mid_flow)
            sml_flow_str = format_capital(sml_flow)
            
            # 计算各类资金占比
            total_flow = abs(super_flow) + abs(big_flow) + abs(mid_flow) + abs(sml_flow)
            if total_flow > 0:
                super_pct = abs(super_flow) / total_flow * 100
                big_pct = abs(big_flow) / total_flow * 100
                mid_pct = abs(mid_flow) / total_flow * 100
                sml_pct = abs(sml_flow) / total_flow * 100
            else:
                super_pct = big_pct = mid_pct = sml_pct = 0
            
            # 活跃度评估（基于总流入流出金额）
            quote = analysis_data.realtime_quote
            turnover_rate = quote.get('turnover_rate', 0)
            
            if turnover_rate > 5:
                activity = "高"
                activity_stars = "★★★★★"
            elif turnover_rate > 3:
                activity = "中高"
                activity_stars = "★★★★☆"
            elif turnover_rate > 1:
                activity = "中等"
                activity_stars = "★★★☆☆"
            elif turnover_rate > 0.5:
                activity = "中低"
                activity_stars = "★★☆☆☆"
            else:
                activity = "低"
                activity_stars = "★☆☆☆☆"
            
            # 格式化时间信息
            time_info = ""
            if latest_flow.capital_flow_item_time:
                time_info = f"数据时间: {latest_flow.capital_flow_item_time}"
            elif latest_flow.last_valid_time:
                time_info = f"更新时间: {latest_flow.last_valid_time}"
            
            # 构建显示文本
            capital_text = (
                f"主力净{main_flow_direction}: [{main_flow_color}]{main_flow_str}[/{main_flow_color}]    "
                f"超大单: {super_flow_str}({super_pct:.1f}%)    大单: {big_flow_str}({big_pct:.1f}%)    "
                f"中单: {mid_flow_str}({mid_pct:.1f}%)    小单: {sml_flow_str}({sml_pct:.1f}%)    │    "
                f"活跃度: {activity}    热度: {activity_stars}"
            )
            
            if time_info:
                capital_text += f"    {time_info}"
            
            # 添加技术指标信息
            tech_text = self._format_technical_indicators_for_money_flow(analysis_data)
            if tech_text:
                capital_text += f"\n{tech_text}"
            
            return capital_text
            
        except Exception as e:
            self.logger.error(f"获取资金流向数据失败: {e}")
            # 发生异常时回退到估算方法
            return self._fallback_estimated_capital_flow(analysis_data)
    
    def _fallback_estimated_capital_flow(self, analysis_data) -> str:
        """回退的估算资金流向方法"""
        quote = analysis_data.realtime_quote
        change_rate = quote.get('change_rate', 0)
        turnover = quote.get('turnover', 0)
        turnover_rate = quote.get('turnover_rate', 0)
        
        # 基于涨跌幅和成交量估算资金流向
        if change_rate > 0:
            main_flow_direction = "流入"
            main_flow_color = "green"
        else:
            main_flow_direction = "流出"
            main_flow_color = "red"
        
        # 估算主力资金（基于成交额的一定比例）
        estimated_main_flow = turnover * 0.3  # 假设主力资金占30%
        
        if estimated_main_flow > 100000000:
            main_flow_str = f"{estimated_main_flow/100000000:.1f}亿"
        elif estimated_main_flow > 10000:
            main_flow_str = f"{estimated_main_flow/10000:.1f}万"
        else:
            main_flow_str = f"{estimated_main_flow:.0f}"
        
        # 活跃度评估
        if turnover_rate > 5:
            activity = "高"
            activity_stars = "★★★★★"
        elif turnover_rate > 3:
            activity = "中高"
            activity_stars = "★★★★☆"
        elif turnover_rate > 1:
            activity = "中等"
            activity_stars = "★★★☆☆"
        elif turnover_rate > 0.5:
            activity = "中低"
            activity_stars = "★★☆☆☆"
        else:
            activity = "低"
            activity_stars = "★☆☆☆☆"
        
        capital_text = (
            f"主力净{main_flow_direction}: [{main_flow_color}]{main_flow_str}[/{main_flow_color}](估算)    "
            f"超大单: 估算中    大单: 估算中    中单: 估算中    小单: 估算中    │    "
            f"活跃度: {activity}    热度: {activity_stars}"
        )
        
        # 添加技术指标信息
        tech_text = self._format_technical_indicators_for_money_flow(analysis_data)
        if tech_text:
            capital_text += f"\n{tech_text}"
        
        return capital_text
    
    def _format_technical_indicators_for_money_flow(self, analysis_data) -> str:
        """格式化技术指标信息用于资金流向区域显示"""
        if not analysis_data or not hasattr(analysis_data, 'technical_indicators') or not analysis_data.technical_indicators:
            return ""
        
        tech_indicators = analysis_data.technical_indicators
        tech_text = "📊 技术指标: "
        
        # MA移动平均线
        ma_parts = []
        for ma_period in ['ma5', 'ma10', 'ma20', 'ma60']:
            if tech_indicators.get(ma_period):
                ma_parts.append(f"{ma_period.upper()}: {tech_indicators[ma_period]:.2f}")
        
        if ma_parts:
            tech_text += "  ".join(ma_parts) + "    "
        
        # RSI指标
        if tech_indicators.get('rsi'):
            rsi_value = tech_indicators['rsi']
            if rsi_value > 70:
                rsi_status = "[red]超买[/red]"
            elif rsi_value < 30:
                rsi_status = "[green]超卖[/green]"
            else:
                rsi_status = "正常"
            tech_text += f"RSI(14): {rsi_value:.1f}({rsi_status})    "
        
        # MACD指标
        if tech_indicators.get('macd') and isinstance(tech_indicators['macd'], dict):
            macd_data = tech_indicators['macd']
            dif = macd_data.get('dif', 0)
            dea = macd_data.get('dea', 0)
            histogram = macd_data.get('histogram', 0)
            
            # 判断金叉死叉
            if dif > dea:
                macd_trend = "[green]金叉[/green]" if histogram > 0 else "[yellow]转强[/yellow]"
            else:
                macd_trend = "[red]死叉[/red]" if histogram < 0 else "[yellow]转弱[/yellow]"
            
            tech_text += f"MACD: DIF({dif:.3f}) DEA({dea:.3f}) 柱({histogram:.3f}) {macd_trend}    "
        
        # 趋势分析
        trend_parts = []
        if tech_indicators.get('price_trend'):
            price_trend = tech_indicators['price_trend']
            if "上升" in price_trend or "上涨" in price_trend:
                trend_parts.append(f"价格: [green]{price_trend}[/green]")
            elif "下降" in price_trend or "下跌" in price_trend:
                trend_parts.append(f"价格: [red]{price_trend}[/red]")
            else:
                trend_parts.append(f"价格: {price_trend}")
        
        if tech_indicators.get('volume_trend'):
            volume_trend = tech_indicators['volume_trend']
            if "放量" in volume_trend or "增加" in volume_trend:
                trend_parts.append(f"成交量: [blue]{volume_trend}[/blue]")
            elif "缩量" in volume_trend or "减少" in volume_trend:
                trend_parts.append(f"成交量: [dim]{volume_trend}[/dim]")
            else:
                trend_parts.append(f"成交量: {volume_trend}")
        
        if trend_parts:
            tech_text += "    ".join(trend_parts)
        
        return tech_text
    
    # ==================== 市场状态检查方法 ====================
    
    async def _check_market_trading_status(self, stock_code: str) -> bool:
        """
        检查股票所在市场是否处于交易时间
        
        Args:
            stock_code: 股票代码 (如 HK.00700, US.AAPL)
            
        Returns:
            bool: True表示处于交易时间，False表示非交易时间
        """
        try:
            # 获取市场状态
            loop = asyncio.get_event_loop()
            market_states = await loop.run_in_executor(
                None, self.futu_market.get_market_state, [stock_code]
            )
            
            if not market_states or len(market_states) == 0:
                self.logger.warning(f"未能获取股票 {stock_code} 的市场状态，默认为非交易时间")
                return False
            
            market_state = market_states[0].market_state
            
            # 定义交易时间的市场状态
            trading_states = {
                'OPEN',           # 开盘
                'TRADING',        # 交易中
                'MORNING',        # 上午时段
                'AFTERNOON',      # 下午时段
                'PRE_MARKET_BEGIN', # 盘前开始
                'AUCTION',        # 集合竞价
                'UNKNOWN_STATUS'  # 未知状态（保守判断为开盘）
            }
            
            # 检查是否处于交易时间
            is_trading = market_state in trading_states
            
            self.logger.info(f"股票 {stock_code} 市场状态: {market_state}, 是否交易时间: {is_trading}")
            
            return is_trading
            
        except Exception as e:
            self.logger.error(f"检查股票 {stock_code} 市场状态失败: {e}")
            # 出错时保守地返回False，避免在非交易时间启动实时更新
            return False
    
    # ==================== 闪烁效果支持方法 ====================
    
    def get_formatted_data_with_flash(self, stock_code: str, data_type: str, formatted_value: str) -> Tuple[str, bool]:
        """
        检测数据变化并返回是否需要闪烁效果
        
        Args:
            stock_code: 股票代码
            data_type: 数据类型 (quote/orderbook/tick/capital)
            formatted_value: 格式化后的值
            
        Returns:
            (最终显示值, 是否需要闪烁)
        """
        try:
            # 创建缓存键
            if stock_code not in self.last_formatted_values:
                self.last_formatted_values[stock_code] = {}
            
            # 检查是否有变化
            last_value = self.last_formatted_values[stock_code].get(data_type)
            has_changed = last_value != formatted_value
            
            # 更新缓存
            self.last_formatted_values[stock_code][data_type] = formatted_value
            
            if has_changed and last_value is not None:
                self.logger.debug(f"数据变化检测: {stock_code}:{data_type} '{last_value[:50]}...' -> '{formatted_value[:50]}...'")
                # 数据有变化，需要闪烁
                flash_value = self._apply_flash_style(formatted_value, data_type)
                return flash_value, True
            else:
                # 数据无变化或首次设置，不闪烁
                return formatted_value, False
                
        except Exception as e:
            self.logger.error(f"检测数据变化失败: {e}")
            return formatted_value, False
    
    def _apply_flash_style(self, value: str, data_type: str) -> str:
        """
        应用闪烁样式
        
        Args:
            value: 原始值
            data_type: 数据类型
            
        Returns:
            应用了闪烁样式的值
        """
        try:
            # 根据数据类型选择不同的闪烁颜色
            if data_type == 'quote':
                # 报价数据使用黄色背景
                return f"[bold yellow on blue]{value}[/bold yellow on blue]"
            elif data_type == 'orderbook':
                # 五档数据使用蓝色背景
                return f"[bold white on blue]{value}[/bold white on blue]"
            elif data_type == 'tick':
                # 逐笔数据使用绿色背景
                return f"[bold white on green]{value}[/bold white on green]"
            elif data_type == 'capital':
                # 资金流向使用紫色背景
                return f"[bold white on magenta]{value}[/bold white on magenta]"
            else:
                # 默认使用蓝色背景
                return f"[bold white on blue]{value}[/bold white on blue]"
                
        except Exception as e:
            self.logger.error(f"应用闪烁样式失败: {e}")
            return value
    
    async def create_flash_restore_task(self, widget, original_value: str, delay: float = 0.5):
        """
        创建闪烁恢复任务
        
        Args:
            widget: 需要恢复的UI组件
            original_value: 原始值
            delay: 延迟时间（秒）
        """
        try:
            # 创建异步任务来恢复正常样式
            asyncio.create_task(self._restore_widget_normal_style(widget, original_value, delay))
            
        except Exception as e:
            self.logger.error(f"创建闪烁恢复任务失败: {e}")
    
    async def _restore_widget_normal_style(self, widget, original_value: str, delay: float):
        """
        恢复组件的正常样式
        
        Args:
            widget: UI组件
            original_value: 原始值
            delay: 延迟时间
        """
        try:
            # 等待指定时间
            await asyncio.sleep(delay)
            
            # 恢复正常样式
            if widget and hasattr(widget, 'update'):
                widget.update(original_value)
                
        except asyncio.CancelledError:
            pass
        except Exception as e:
            self.logger.error(f"恢复组件正常样式失败: {e}")
    






import pandas as pd

from strategies import Strategies
from utils import logger
from utils.global_vars import get_logger
pd.options.mode.chained_assignment = None  # default='warn'


class RSIThreshold(Strategies):
    def __init__(self, input_data: dict, rsi_1=6, rsi_2=12, rsi_3=24, lower_rsi=30, upper_rsi=70, observation=100):
        """
        Initialize RSI-Threshold Strategy Instance
        :param input_data:
        :param rsi_1: RSI Period 1 (Default = 6)
        :param rsi_2: RSI Period 2 (Default = 12)
        :param rsi_3: RSI Period 3 (Default = 24)
        :param lower_rsi: Lower RSI Threshold (Default = 30)
        :param upper_rsi: Upper RSI Threshold (Default = 70)
        :param observation: Observation Period in Dataframe (Default = 100)
        """
        self.RSI_1 = rsi_1
        self.RSI_2 = rsi_2
        self.RSI_3 = rsi_3
        self.LOWER_RSI = lower_rsi
        self.UPPER_RSI = upper_rsi
        self.OBSERVATION = observation
        self.default_logger = get_logger("rsi_threshold")

        super().__init__(input_data)
        self.parse_data()

    def __compute_RSI(self, stock_code, time_window):
        diff = self.input_data[stock_code]['close'].diff(1).dropna()  # diff in one field(one day)

        # this preservers dimensions off diff values
        up_chg = 0 * diff
        down_chg = 0 * diff

        # up change is equal to the positive difference, otherwise equal to zero
        up_chg[diff > 0] = diff[diff > 0]

        # down change is equal to negative difference, otherwise equal to zero
        down_chg[diff < 0] = diff[diff < 0]

        # check pandas documentation for ewm
        # https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.ewm.html
        # values are related to exponential decay
        # we set com=time_window-1 so we get decay alpha=1/time_window
        up_chg_avg = up_chg.ewm(com=time_window - 1, min_periods=time_window).mean()
        down_chg_avg = down_chg.ewm(com=time_window - 1, min_periods=time_window).mean()

        rs = abs(up_chg_avg / down_chg_avg)
        rsi = 100 - 100 / (1 + rs)
        return rsi

    def parse_data(self, stock_list: list = None, latest_data: pd.DataFrame = None, backtesting: bool = False):
        # Received New Data => Parse it Now to input_data
        if latest_data is not None:
            # Only need to update MACD for the stock_code with new data
            stock_list = [latest_data['code'][0]]

            # Remove records with duplicate time_key. Always use the latest data to override
            time_key = latest_data['time_key'][0]
            self.input_data[stock_list[0]].drop(
                self.input_data[stock_list[0]][self.input_data[stock_list[0]].time_key == time_key].index,
                inplace=True)
            # Append empty columns and concat at the bottom
            latest_data = pd.concat([latest_data, pd.DataFrame(columns=['rsi_1', 'rsi_2', 'rsi_3'])])
            self.input_data[stock_list[0]] = pd.concat([self.input_data[stock_list[0]], latest_data])
        elif stock_list is not None:
            # Override Updated Stock List
            stock_list = stock_list
        else:
            stock_list = self.input_data.keys()

        # Calculate EMA for the stock_list
        for stock_code in stock_list:
            # Need to truncate to a maximum length for low-latency
            if not backtesting:
                self.input_data[stock_code] = self.input_data[stock_code].iloc[
                                              -min(self.OBSERVATION, self.input_data[stock_code].shape[0]):]
            self.input_data[stock_code][['open', 'close', 'high', 'low']] = self.input_data[stock_code][
                ['open', 'close', 'high', 'low']].apply(pd.to_numeric)

            self.input_data[stock_code]['rsi_1'] = self.__compute_RSI(stock_code=stock_code, time_window=self.RSI_1)
            self.input_data[stock_code]['rsi_2'] = self.__compute_RSI(stock_code=stock_code, time_window=self.RSI_2)
            self.input_data[stock_code]['rsi_3'] = self.__compute_RSI(stock_code=stock_code, time_window=self.RSI_3)

            self.input_data[stock_code].reset_index(drop=True, inplace=True)

    def buy(self, stock_code) -> bool:
        current_record, previous_record = self.get_current_and_previous_record(stock_code)
        # Buy Decision based on RSI值超过了超卖线
        buy_decision = current_record['rsi_1'] < self.LOWER_RSI < previous_record['rsi_1']

        if buy_decision:
            self.default_logger.info(
                f"Buy Decision: {current_record['time_key']} based on \n {pd.concat([previous_record.to_frame().transpose(), current_record.to_frame().transpose()], axis=0)}")

        return buy_decision

    def sell(self, stock_code) -> bool:
        current_record, previous_record = self.get_current_and_previous_record(stock_code)
        # Sell Decision based on RSI值超过了超买线
        sell_decision = current_record['rsi_1'] > self.UPPER_RSI > previous_record['rsi_1']

        if sell_decision:
            self.default_logger.info(
                f"Sell Decision: {current_record['time_key']} based on \n {pd.concat([previous_record.to_frame().transpose(), current_record.to_frame().transpose()], axis=0)}")
        return sell_decision



from dataclasses import dataclass
from typing import Optional

from textual import on
from textual.app import ComposeResult
from textual.binding import Binding
from textual.message import Message
from textual.widget import Widget
from textual.widgets import Static, Button, Input, TextArea
from textual.containers import Horizontal, Vertical, Center


class ConfirmDialog(Widget, can_focus_children=True):
    """确认对话框组件
    
    用于显示确认信息并等待用户确认或取消的对话框
    """
    
    DEFAULT_CSS = """
    ConfirmDialog {
        layout: vertical;
        dock: top; 
        padding: 2;                       
        width: 60%;
        height: auto;
        max-height: 50%;
        display: none;
        background: $surface;
        border: thick $primary;
        border-title-color: $text;
        margin: 2 10;
        
        &.visible {
            display: block;
        }
        
        .message-content {
            height: auto;
            text-align: center;
            padding: 1 2;
            color: $text;
        }
        
        Center {
            width: 100%;
            height: auto;
            margin-top: 1;
        }
        
        .button-container {
            layout: horizontal;
            height: auto;
            width: auto;
            align: center middle;
        }
        
        .button-container Button {
            margin: 0 1;
            min-width: 12;
        }
        
        Button.-confirm {
            background: $success;
            color: $text;
        }
        
        Button.-cancel {
            background: $error;
            color: $text;
        }
    }    
    """

    DEFAULT_CLASSES = "float"
    BORDER_TITLE = "确认"

    # 键盘绑定
    BINDINGS = [
        Binding("enter", "confirm", "确认", priority=True),
        Binding("escape", "cancel", "取消", priority=True),
        Binding("y", "confirm", "确认"),
        Binding("n", "cancel", "取消"),
    ]

    @dataclass
    class Confirm(Message):
        """确认消息"""
        confirmed: bool = True

    class Cancel(Message):
        """取消消息"""
        pass

    def __init__(self, message: str, title: Optional[str] = None, 
                 confirm_text: str = "确认", cancel_text: str = "取消") -> None:
        """初始化确认对话框
        
        Args:
            message: 要显示的确认信息
            title: 对话框标题，默认为"确认"
            confirm_text: 确认按钮文本
            cancel_text: 取消按钮文本
        """
        super().__init__()
        
        self.message = message
        self.confirm_text = confirm_text
        self.cancel_text = cancel_text
        self._dialog_title = title

    def compose(self) -> ComposeResult:
        """构建确认对话框UI"""
        # 消息显示区域
        yield Static(self.message, classes="message-content")
        
        # 按钮容器 - 使用Center容器确保居中
        with Center():
            with Horizontal(classes="button-container"):
                yield Button(self.confirm_text, variant="success", id="confirm-btn")
                yield Button(self.cancel_text, variant="error", id="cancel-btn")

    def on_mount(self) -> None:
        """组件挂载时的初始化"""
        # 设置标题
        if self._dialog_title:
            self.border_title = self._dialog_title
        
        # 自动聚焦到确认按钮
        try:
            confirm_btn = self.query_one("#confirm-btn", Button)
            confirm_btn.focus()
        except Exception:
            pass

    @on(Button.Pressed, "#confirm-btn")
    def on_confirm_pressed(self, event: Button.Pressed) -> None:
        """处理确认按钮点击"""
        event.stop()
        self.action_confirm()

    @on(Button.Pressed, "#cancel-btn")
    def on_cancel_pressed(self, event: Button.Pressed) -> None:
        """处理取消按钮点击"""
        event.stop()
        self.action_cancel()

    def action_confirm(self) -> None:
        """确认操作"""
        self.post_message(self.Confirm(confirmed=True))

    def action_cancel(self) -> None:
        """取消操作"""
        self.post_message(self.Cancel())

    def allow_focus_children(self) -> bool:
        """只有在可见时才允许子组件获得焦点"""
        return self.has_class("visible")

    def show(self) -> None:
        """显示对话框"""
        self.add_class("visible")
        # 聚焦到确认按钮
        try:
            confirm_btn = self.query_one("#confirm-btn", Button)
            confirm_btn.focus()
        except Exception:
            pass

    def hide(self) -> None:
        """隐藏对话框"""
        self.remove_class("visible")


class InputDialog(Widget, can_focus_children=True):
    """输入对话框组件
    
    用于获取用户文本输入的对话框
    """
    
    DEFAULT_CSS = """
    InputDialog {
        layout: vertical;
        dock: top; 
        padding: 2;                       
        width: 80%;
        height: auto;
        max-height: 60%;
        display: none;
        background: $surface;
        border: thick $primary;
        border-title-color: $text;
        margin: 2 10;
        
        &.visible {
            display: block;
        }
        
        .input-content {
            layout: vertical;
            height: auto;
            padding: 1 2;
        }
        
        .input-label {
            height: auto;
            text-align: left;
            padding: 0 0 1 0;
            color: $text;
        }
        
        .input-field {
            height: 8;
            width: 100%;
            border: solid $border;
            padding: 1;
        }
        
        Center {
            width: 100%;
            height: auto;
            margin-top: 1;
        }
        
        .button-container {
            layout: horizontal;
            height: auto;
            width: auto;
            align: center middle;
        }
        
        .button-container Button {
            margin: 0 1;
            min-width: 12;
        }
        
        Button.-confirm {
            background: $success;
            color: $text;
        }
        
        Button.-cancel {
            background: $error;
            color: $text;
        }
    }    
    """

    DEFAULT_CLASSES = "float"
    BORDER_TITLE = "输入"

    # 键盘绑定
    BINDINGS = [
        Binding("ctrl+enter", "submit", "提交", priority=True),
        Binding("escape", "cancel", "取消", priority=True),
    ]

    @dataclass
    class Submit(Message):
        """提交消息"""
        text: str = ""

    class Cancel(Message):
        """取消消息"""
        pass

    def __init__(self, prompt: str = "请输入内容:", title: Optional[str] = None, 
                 placeholder: str = "", multiline: bool = True) -> None:
        """初始化输入对话框
        
        Args:
            prompt: 提示信息
            title: 对话框标题，默认为"输入"
            placeholder: 输入框占位符
            multiline: 是否支持多行输入
        """
        super().__init__()
        
        self.prompt = prompt
        self.placeholder = placeholder
        self.multiline = multiline
        self._dialog_title = title
        self.input_text = ""

    def compose(self) -> ComposeResult:
        """构建输入对话框UI"""
        with Vertical(classes="input-content"):
            # 提示信息
            yield Static(self.prompt, classes="input-label")
            
            # 输入框 - 根据multiline选择TextArea或Input
            if self.multiline:
                yield TextArea(
                    classes="input-field", 
                    id="input-field"
                )
            else:
                yield Input(placeholder=self.placeholder,
                           classes="input-field", 
                           id="input-field")
        
        # 按钮容器
        with Center():
            with Horizontal(classes="button-container"):
                yield Button("提交", variant="success", id="submit-btn")
                yield Button("取消", variant="error", id="cancel-btn")

    def on_mount(self) -> None:
        """组件挂载时的初始化"""
        # 设置标题
        if self._dialog_title:
            self.border_title = self._dialog_title
        
        # 自动聚焦到输入框
        try:
            input_field = self.query_one("#input-field")
            
            # 如果是TextArea且有占位符文本，设置初始文本
            if self.multiline and hasattr(input_field, 'text') and self.placeholder:
                input_field.text = f"# {self.placeholder}\n\n"
            
            input_field.focus()
        except Exception:
            pass

    @on(Button.Pressed, "#submit-btn")
    def on_submit_pressed(self, event: Button.Pressed) -> None:
        """处理提交按钮点击"""
        event.stop()
        self.action_submit()

    @on(Button.Pressed, "#cancel-btn")
    def on_cancel_pressed(self, event: Button.Pressed) -> None:
        """处理取消按钮点击"""
        event.stop()
        self.action_cancel()

    def action_submit(self) -> None:
        """提交操作"""
        try:
            input_field = self.query_one("#input-field")
            if self.multiline and hasattr(input_field, 'text'):
                text = input_field.text
                # 如果是TextArea，过滤掉占位符文本
                if text.startswith(f"# {self.placeholder}\n\n"):
                    text = text[len(f"# {self.placeholder}\n\n"):]
            elif hasattr(input_field, 'value'):
                text = input_field.value
            else:
                text = ""
            
            self.post_message(self.Submit(text=text.strip()))
        except Exception:
            self.post_message(self.Submit(text=""))

    def action_cancel(self) -> None:
        """取消操作"""
        self.post_message(self.Cancel())

    def allow_focus_children(self) -> bool:
        """只有在可见时才允许子组件获得焦点"""
        return self.has_class("visible")

    def show(self) -> None:
        """显示对话框"""
        self.add_class("visible")
        # 聚焦到输入框
        try:
            input_field = self.query_one("#input-field")
            input_field.focus()
        except Exception:
            pass

    def hide(self) -> None:
        """隐藏对话框"""
        self.remove_class("visible")


class AIInputDialog(InputDialog):
    """AI交互专用输入对话框"""
    
    DEFAULT_CLASSES = "float"
    BORDER_TITLE = "💬 AI 智能助手"

    def __init__(self, **kwargs):
        """初始化AI输入对话框"""
        super().__init__(
            prompt="请输入您想要咨询的问题:",
            title="💬 AI 智能助手",
            placeholder="例如: 请分析一下这只股票的投资价值...",
            multiline=True,
            **kwargs
        )


"""
UIManager - UI组件和界面状态管理模块

负责UI组件引用管理、界面更新、表格操作和光标控制
"""

import asyncio
from typing import Optional, Any

from textual.widgets import DataTable, Static
# StockData导入已移除，未在此文件中使用
from utils.global_vars import get_logger


class UIManager:
    """
    UI管理器
    负责UI组件和界面状态管理
    """
    
    def __init__(self, app_core, app_instance):
        """初始化UI管理器"""
        self.app_core = app_core
        self.app = app_instance
        self.logger = get_logger(__name__)
        
        # UI组件引用
        self.stock_table: Optional[DataTable] = None
        self.group_table: Optional[DataTable] = None
        self.orders_table: Optional[DataTable] = None
        self.position_table: Optional[DataTable] = None  # 持仓信息表格
        self.trading_mode_display: Optional[Static] = None
        self.chart_panel: Optional[Static] = None
        self.ai_analysis_panel: Optional[Static] = None
        self.info_panel: Optional[Any] = None

        # 保留兼容性：position_content 和 group_stocks_content 指向 position_table
        self.position_content: Optional[DataTable] = None
        self.group_stocks_content: Optional[DataTable] = None
        
        # 缓存上次的单元格值，用于检测变化
        self.last_cell_values: dict = {}
        
        # 状态栏组件引用
        self.connection_status: Optional[Static] = None
        self.market_status: Optional[Static] = None
        self.refresh_mode: Optional[Static] = None
        self.last_update: Optional[Static] = None
        
        self.logger.info("UIManager 初始化完成")
    
    async def setup_ui_references(self) -> None:
        """设置UI组件引用"""
        # 获取股票表格组件
        try:
            self.stock_table = self.app.query_one("#stock_table", DataTable)
            self.stock_table.cursor_type = 'row'
            # 默认激活股票表格光标
            self.stock_table.show_cursor = True
            self.logger.debug("股票表格引用设置成功")
        except Exception as e:
            self.logger.error(f"获取股票表格引用失败: {e}")
        
        # 获取用户分组相关组件
        try:
            self.group_table = self.app.query_one("#group_table", DataTable)
            # 配置分组表格的光标特性
            if self.group_table:
                self.group_table.cursor_type = "row"
                # 默认不显示分组表格光标
                self.group_table.show_cursor = False
            self.logger.debug("分组表格引用设置成功")
        except Exception as e:
            self.logger.error(f"获取分组表格引用失败: {e}")

        # 获取交易模式显示组件
        try:
            self.trading_mode_display = self.app.query_one("#trading_mode_display", Static)
            self.logger.debug("交易模式显示组件引用设置成功")
        except Exception as e:
            self.logger.error(f"获取交易模式显示组件引用失败: {e}")

        # 获取持仓信息表格组件
        try:
            self.position_table = self.app.query_one("#position_table", DataTable)
            # 配置持仓表格的光标特性
            if self.position_table:
                self.position_table.cursor_type = "row"
                # 默认不显示持仓表格光标
                self.position_table.show_cursor = False
            # 兼容性：position_content 和 group_stocks_content 指向同一个组件
            self.position_content = self.position_table
            self.group_stocks_content = self.position_table
            self.logger.debug("持仓信息表格引用设置成功")
        except Exception as e:
            self.logger.error(f"获取持仓信息表格引用失败: {e}")

        # 获取订单表格组件
        try:
            self.orders_table = self.app.query_one("#orders_table", DataTable)
            # 配置订单表格的光标特性
            if self.orders_table:
                self.orders_table.cursor_type = "row"
                # 默认不显示订单表格光标
                self.orders_table.show_cursor = False
            self.logger.debug("订单表格引用设置成功")
        except Exception as e:
            self.logger.error(f"获取订单表格引用失败: {e}")
        
        # 获取图表面板（可能在分析界面标签页中）
        try:
            self.chart_panel = self.app.query_one("#kline_content", Static)
            self.logger.debug("图表面板引用设置成功")
        except Exception as e:
            self.logger.debug(f"图表面板不在当前标签页中: {e}")
            self.chart_panel = None
        
        # 获取AI分析面板（可能在分析界面标签页中）
        try:
            self.ai_analysis_panel = self.app.query_one("#ai_analysis_content", Static)
            self.logger.debug("AI分析面板引用设置成功")
        except Exception as e:
            self.logger.debug(f"AI分析面板不在当前标签页中: {e}")
            self.ai_analysis_panel = None
        
        # 获取InfoPanel引用
        try:
            from monitor.widgets.line_panel import InfoPanel
            self.info_panel = self.app.query_one("#info_panel", InfoPanel)
            self.logger.info("InfoPanel引用设置成功")

            # 设置交易管理器
            if hasattr(self.app, 'futu_trade'):
                self.info_panel.set_trade_manager(self.app.futu_trade)
                self.logger.info("InfoPanel交易管理器设置成功")
        except Exception as e:
            self.logger.error(f"获取InfoPanel引用失败: {e}")
        
        # 获取状态栏组件引用
        try:
            self.connection_status = self.app.query_one("#connection_status", Static)
            self.logger.debug("连接状态组件引用设置成功")
        except Exception as e:
            self.logger.error(f"获取连接状态组件引用失败: {e}")
            
        try:
            self.market_status = self.app.query_one("#market_status", Static)
            self.logger.debug("市场状态组件引用设置成功") 
        except Exception as e:
            self.logger.error(f"获取市场状态组件引用失败: {e}")
            
        try:
            self.refresh_mode = self.app.query_one("#refresh_mode", Static)
            self.logger.debug("刷新模式组件引用设置成功")
        except Exception as e:
            self.logger.error(f"获取刷新模式组件引用失败: {e}")
            
            
        try:
            self.last_update = self.app.query_one("#last_update", Static)
            self.logger.debug("最后更新时间组件引用设置成功")
        except Exception as e:
            self.logger.error(f"获取最后更新时间组件引用失败: {e}")
        
        self.logger.info("UI组件引用设置完成")
    
    async def initialize_info(self) -> None:
        """初始化InfoPanel"""
        try:
            if self.info_panel:
                from monitor.widgets.line_panel import InfoType, InfoLevel
                # 添加启动信息
                await self.info_panel.log_info("应用程序启动成功", "系统")
                await self.info_panel.log_info(f"监控股票数量: {len(self.app_core.monitored_stocks)}", "系统")
                await self.info_panel.log_info(f"连接状态: {self.app_core.connection_status.value}", "系统")

                # 添加操作提示
                await self.info_panel.add_info(
                    "使用快捷键:  A-左 D-右 Q-退出",
                    InfoType.USER_ACTION,
                    InfoLevel.INFO,
                    "系统提示"
                )

                self.logger.info("InfoPanel 初始化完成")

            # 初始化交易模式显示
            await self.update_trading_mode_display()

        except Exception as e:
            self.logger.error(f"初始化InfoPanel失败: {e}")
    
    async def load_default_stocks(self) -> None:
        """加载默认股票到表格"""
        if self.stock_table:
            # 清空现有数据
            self.stock_table.clear()
            
            # 添加股票行
            for stock_code in self.app_core.monitored_stocks:
                self.stock_table.add_row(
                    stock_code,
                    stock_code,
                    "0.00",
                    "0.00%",
                    "0",
                    "未更新",
                    key=stock_code
                )
        
        self.logger.info(f"加载默认股票列表: {self.app_core.monitored_stocks}")
        if self.info_panel:
            await self.info_panel.log_info(f"加载默认股票列表: {self.app_core.monitored_stocks}", "系统")
        
        # 初始化股票光标位置
        self.app_core.current_stock_cursor = 0
        #if self.stock_table and len(self.app_core.monitored_stocks) > 0:
        #    await self.update_stock_cursor()
        
        # 初始化表格焦点状态
        await self.update_table_focus()
    
    async def update_stock_table(self) -> None:
        """更新股票表格"""
        if not self.stock_table:
            self.logger.warning("股票表格引用为空，无法更新")
            return
        
        try:
            updated_count = 0
            # 更新表格数据
            for stock_code in self.app_core.monitored_stocks:
                stock_info = self.app_core.stock_data.get(stock_code)
                self.logger.debug(f'UI股票数据: {stock_code} {stock_info}')
                if stock_info:
                    # 格式化数据
                    price_str = f"{stock_info.current_price:.2f}"
                    change_str = f"{stock_info.change_rate:.2f}%"
                    volume_str = f"{stock_info.volume:,}"
                    time_str = stock_info.update_time.strftime("%H:%M:%S")
                    
                    self.logger.debug(f'UI更新股票数据: {stock_code} - {stock_info.name} {price_str} {change_str}')
                    
                    
                    self.stock_table.update_cell(stock_code,'name', stock_info.name)
                    self.stock_table.update_cell(stock_code,'time', time_str)

                    # 更新行数据 - 先应用闪烁效果
                    await self.update_cell_with_flash(stock_code, 'price', price_str, 
                                                    change_rate=stock_info.change_rate)
                    await self.update_cell_with_flash(stock_code, 'change', change_str, 
                                                    change_rate=stock_info.change_rate)
                    await self.update_cell_with_flash(stock_code, 'volume', volume_str)
                    
                    updated_count += 1
                else:
                    self.logger.warning(f"股票 {stock_code} 没有数据，跳过更新")
            
            # 强制刷新表格显示
            #self.stock_table.refresh()
            self.logger.info(f"股票表格更新完成，共更新 {updated_count} 只股票")
                    
        except Exception as e:
            self.logger.error(f"更新股票表格失败: {e}")
            # 尝试强制刷新以确保UI同步
            if self.stock_table:
                self.stock_table.refresh()
    
    async def update_stock_cursor(self) -> None:
        """更新股票表格的光标显示"""
        if not self.stock_table or len(self.app_core.monitored_stocks) == 0:
            return
            
        try:
            # 确保光标位置在有效范围内
            if self.app_core.current_stock_cursor < 0:
                self.app_core.current_stock_cursor = 0
            elif self.app_core.current_stock_cursor >= len(self.app_core.monitored_stocks):
                self.app_core.current_stock_cursor = len(self.app_core.monitored_stocks) - 1
            
            # 使用DataTable的原生光标移动功能
            self.stock_table.move_cursor(
                row=self.app_core.current_stock_cursor, 
                column=0,
                animate=False,
                scroll=True
            )
            
            # 更新当前选中的股票代码
            if 0 <= self.app_core.current_stock_cursor < len(self.app_core.monitored_stocks):
                self.app_core.current_stock_code = self.app_core.monitored_stocks[self.app_core.current_stock_cursor]

                #self.logger.debug(f"股票光标移动到行 {self.app_core.current_stock_cursor}, 股票: {self.app_core.current_stock_code}")

        except Exception as e:
            self.logger.error(f"更新股票光标失败: {e}")
            # 降级处理：仅更新当前股票代码
            if 0 <= self.app_core.current_stock_cursor < len(self.app_core.monitored_stocks):
                self.app_core.current_stock_code = self.app_core.monitored_stocks[self.app_core.current_stock_cursor]
    
    async def update_group_cursor(self) -> None:
        """更新分组表格的光标显示 - 使用DataTable原生光标"""
        if not self.group_table or len(self.app_core.group_data) == 0:
            return

        try:
            # 确保光标位置在有效范围内
            if self.app_core.current_group_cursor < 0:
                self.app_core.current_group_cursor = 0
            elif self.app_core.current_group_cursor >= len(self.app_core.group_data):
                self.app_core.current_group_cursor = len(self.app_core.group_data) - 1

            # 使用DataTable的原生光标移动功能
            self.group_table.move_cursor(
                row=self.app_core.current_group_cursor,
                column=0,
                animate=False,
                scroll=True
            )

            self.logger.debug(f"分组光标移动到行 {self.app_core.current_group_cursor}")

        except Exception as e:
            self.logger.error(f"更新分组光标失败: {e}")

    async def update_position_cursor(self) -> None:
        """更新持仓表格的光标显示 - 使用DataTable原生光标"""
        if not self.position_table or len(self.app_core.position_data) == 0:
            return

        try:
            # 确保光标位置在有效范围内
            if self.app_core.current_position_cursor < 0:
                self.app_core.current_position_cursor = 0
            elif self.app_core.current_position_cursor >= len(self.app_core.position_data):
                self.app_core.current_position_cursor = len(self.app_core.position_data) - 1

            # 使用DataTable的原生光标移动功能
            self.position_table.move_cursor(
                row=self.app_core.current_position_cursor,
                column=0,
                animate=False,
                scroll=True
            )

            self.logger.debug(f"持仓光标移动到行 {self.app_core.current_position_cursor}")

        except Exception as e:
            self.logger.error(f"更新持仓光标失败: {e}")

    async def update_order_cursor(self) -> None:
        """更新订单表格的光标显示 - 使用DataTable原生光标"""
        if not self.orders_table or len(self.app_core.order_data) == 0:
            return

        try:
            # 确保光标位置在有效范围内
            if self.app_core.current_order_cursor < 0:
                self.app_core.current_order_cursor = 0
            elif self.app_core.current_order_cursor >= len(self.app_core.order_data):
                self.app_core.current_order_cursor = len(self.app_core.order_data) - 1

            # 使用DataTable的原生光标移动功能
            self.orders_table.move_cursor(
                row=self.app_core.current_order_cursor,
                column=0,
                animate=False,
                scroll=True
            )

            self.logger.debug(f"订单光标移动到行 {self.app_core.current_order_cursor}")

        except Exception as e:
            self.logger.error(f"更新订单光标失败: {e}")

    async def update_table_focus(self) -> None:
        """更新表格焦点显示，确保同一时间只有一个表格显示光标"""
        try:
            if self.app_core.active_table == "stock":
                # 激活股票表格光标，隐藏其他表格光标
                if self.stock_table:
                    self.stock_table.show_cursor = True
                    await self.update_stock_cursor()
                if self.group_table:
                    self.group_table.show_cursor = False
                    self.group_table.refresh()
                if self.position_table:
                    self.position_table.show_cursor = False
                    self.position_table.refresh()
                if self.orders_table:
                    self.orders_table.show_cursor = False
                    self.orders_table.refresh()
                self.logger.debug("激活股票表格焦点")

            elif self.app_core.active_table == "group":
                # 激活分组表格光标，隐藏其他表格光标
                if self.group_table:
                    self.group_table.show_cursor = True
                    await self.update_group_cursor()
                if self.stock_table:
                    self.stock_table.show_cursor = False
                    self.stock_table.refresh()
                if self.position_table:
                    self.position_table.show_cursor = False
                    self.position_table.refresh()
                if self.orders_table:
                    self.orders_table.show_cursor = False
                    self.orders_table.refresh()
                self.logger.debug("激活分组表格焦点")

            elif self.app_core.active_table == "position":
                # 激活持仓表格光标，隐藏其他表格光标
                if self.position_table:
                    self.position_table.show_cursor = True
                    await self.update_position_cursor()
                if self.stock_table:
                    self.stock_table.show_cursor = False
                    self.stock_table.refresh()
                if self.group_table:
                    self.group_table.show_cursor = False
                    self.group_table.refresh()
                if self.orders_table:
                    self.orders_table.show_cursor = False
                    self.orders_table.refresh()
                self.logger.debug("激活持仓表格焦点")

            elif self.app_core.active_table == "orders":
                # 激活订单表格光标，隐藏其他表格光标
                if self.orders_table:
                    self.orders_table.show_cursor = True
                    await self.update_order_cursor()
                if self.stock_table:
                    self.stock_table.show_cursor = False
                    self.stock_table.refresh()
                if self.group_table:
                    self.group_table.show_cursor = False
                    self.group_table.refresh()
                if self.position_table:
                    self.position_table.show_cursor = False
                    self.position_table.refresh()
                self.logger.debug("激活订单表格焦点")

        except Exception as e:
            self.logger.error(f"更新表格焦点失败: {e}")
    
    async def update_group_preview(self) -> None:
        """更新统一窗口中的分组股票信息"""
        try:
            if 0 <= self.app_core.current_group_cursor < len(self.app_core.group_data):
                current_group = self.app_core.group_data[self.app_core.current_group_cursor]
                self.logger.debug(f"当前选中分组: {current_group['name']}")
            else:
                self.logger.debug("无效的分组光标位置")

        except Exception as e:
            self.logger.error(f"更新分组信息失败: {e}")

    async def update_trading_mode_display(self) -> None:
        """更新交易模式显示"""
        try:
            if self.trading_mode_display:
                # 从 data_manager 获取当前交易模式
                data_manager = getattr(self.app_core, 'data_manager', None)
                if data_manager:
                    current_mode = data_manager.get_trading_mode()
                    is_simulation = data_manager.is_simulation_mode()

                    # 根据交易模式设置不同的显示样式
                    if is_simulation:
                        display_text = "[bold yellow]🔄 当前交易模式: 模拟交易[/bold yellow]"
                    else:
                        display_text = "[bold red]⚠️ 当前交易模式: 真实交易[/bold red]"

                    self.trading_mode_display.update(display_text)
                    self.logger.debug(f"交易模式显示已更新: {current_mode}")
                else:
                    self.trading_mode_display.update("[dim]交易模式: 未知[/dim]")
                    self.logger.warning("无法获取 data_manager，交易模式显示为未知")
        except Exception as e:
            self.logger.error(f"更新交易模式显示失败: {e}")

    async def update_orders_table(self) -> None:
        """从 app_core.order_data 直接更新订单表格UI"""
        try:
            if not self.orders_table:
                self.logger.warning("orders_table 未初始化，跳过更新")
                return

            self.logger.info(f"开始更新订单表格，当前有 {len(self.app_core.order_data)} 条订单")

            # 调试：打印order_data的前2条数据
            if self.app_core.order_data:
                for i, order in enumerate(self.app_core.order_data[:2]):
                    self.logger.debug(f"订单数据[{i}]: {order}")

            # 清空现有表格数据，但保留列定义
            self.orders_table.clear(columns=False)
            self.logger.debug("订单表格已清空(保留列定义)")

            # 从 app_core.order_data 读取并更新表格
            for order in self.app_core.order_data:
                try:
                    # 提取订单信息
                    order_id = order.get('order_id', '')
                    stock_code = order.get('code', '')
                    trd_side = order.get('trd_side', '')
                    order_status = order.get('order_status', '')
                    price = order.get('price', 0)
                    qty = order.get('qty', 0)

                    # 格式化价格显示
                    if isinstance(price, (int, float)):
                        price_display = f"{price:.2f}"
                    else:
                        price_display = str(price)

                    # 格式化数量显示
                    qty_display = str(int(float(qty))) if qty else "0"

                    # 转换交易方向并设置颜色
                    if trd_side == 'BUY':
                        order_type = "买入"
                        type_display = f"[green]{order_type}[/green]"
                    elif trd_side == 'SELL':
                        order_type = "卖出"
                        type_display = f"[red]{order_type}[/red]"
                    else:
                        order_type = trd_side
                        type_display = order_type

                    # 转换订单状态并设置颜色
                    status_map = {
                        'WAITING_SUBMIT': '待提交',
                        'SUBMITTING': '提交中',
                        'SUBMITTED': '已提交',
                        'FILLED_PART': '部分成交',
                        'FILLED_ALL': '全部成交',
                        'CANCELLED_PART': '部分撤销',
                        'CANCELLED_ALL': '全部撤销',
                        'FAILED': '失败',
                        'DISABLED': '已失效',
                        'ERROR': '错误'
                    }

                    status_display_text = status_map.get(order_status, order_status)

                    # 根据状态设置不同颜色
                    if order_status in ['FILLED_ALL', 'FILLED_PART']:
                        status_display = f"[green]{status_display_text}[/green]"
                    elif order_status in ['SUBMITTED', 'WAITING_SUBMIT', 'SUBMITTING']:
                        status_display = f"[yellow]{status_display_text}[/yellow]"
                    elif order_status in ['CANCELLED_PART', 'CANCELLED_ALL', 'FAILED', 'DISABLED', 'ERROR']:
                        status_display = f"[red]{status_display_text}[/red]"
                    else:
                        status_display = status_display_text

                    # 添加到表格
                    display_order_id = order_id[-8:] if len(order_id) > 8 else order_id
                    self.logger.debug(f"添加订单行: {display_order_id} {stock_code} {order_type} {status_display_text} {price_display} {qty_display}")

                    self.orders_table.add_row(
                        display_order_id,  # 显示订单号后8位
                        stock_code,
                        type_display,
                        status_display,
                        price_display,  # 价格
                        qty_display,    # 数量
                        key=order_id
                    )

                    self.logger.debug(f"订单行添加成功: {order_id}")

                except Exception as e:
                    self.logger.error(f"处理订单UI显示失败: {e}, 订单数据: {order}")
                    import traceback
                    self.logger.error(f"错误堆栈: {traceback.format_exc()}")
                    continue

            # 检查表格行数
            table_row_count = self.orders_table.row_count
            self.logger.info(f"订单表格UI更新完成，app_core有 {len(self.app_core.order_data)} 条订单，表格显示 {table_row_count} 行")

            # 强制刷新表格显示
            self.orders_table.refresh()
            self.logger.debug("订单表格已强制刷新显示")

        except Exception as e:
            self.logger.error(f"更新订单表格失败: {e}")
            import traceback
            self.logger.error(f"详细错误: {traceback.format_exc()}")

    async def update_position_table(self) -> None:
        """从 app_core.position_data 直接更新持仓表格UI"""
        try:
            if not self.position_table:
                self.logger.warning("position_table 未初始化，跳过更新")
                return

            self.logger.info(f"开始更新持仓表格，当前有 {len(self.app_core.position_data)} 只持仓")

            # 清空现有表格数据，但保留列定义
            self.position_table.clear(columns=False)
            self.logger.debug("持仓表格已清空(保留列定义)")

            # 从 app_core.position_data 读取并更新表格
            for position in self.app_core.position_data:
                try:
                    # 打印完整的持仓数据以便调试
                    self.logger.debug(f"持仓原始数据: {position}")

                    # 提取持仓信息
                    stock_code = position.get('stock_code', '')
                    stock_name = position.get('stock_name', '')
                    qty = str(int(position.get('qty', 0)))
                    can_sell_qty = str(int(position.get('can_sell_qty', 0)))
                    cost_price = f"{position.get('cost_price', 0):.3f}"
                    nominal_price = f"{position.get('nominal_price', 0):.3f}"
                    pl_val = position.get('pl_val', 0)
                    pl_ratio = position.get('pl_ratio', 0)

                    self.logger.debug(f"提取的字段 - stock_code: '{stock_code}', stock_name: '{stock_name}', qty: {qty}")

                    # 格式化盈亏显示并设置颜色
                    pl_val_str = f"{pl_val:+,.2f}"
                    pl_ratio_str = f"{pl_ratio:+.2f}%"

                    if pl_val >= 0:
                        pl_val_display = f"[green]{pl_val_str}[/green]"
                        pl_ratio_display = f"[green]{pl_ratio_str}[/green]"
                    else:
                        pl_val_display = f"[red]{pl_val_str}[/red]"
                        pl_ratio_display = f"[red]{pl_ratio_str}[/red]"

                    # 添加到表格
                    self.logger.debug(f"添加持仓行: {stock_code} {stock_name} {qty} {cost_price}")

                    self.position_table.add_row(
                        stock_code,
                        stock_name,
                        qty,
                        can_sell_qty,
                        cost_price,
                        nominal_price,
                        pl_val_display,
                        pl_ratio_display,
                        key=stock_code
                    )

                    self.logger.debug(f"持仓行添加成功: {stock_code}")

                except Exception as e:
                    self.logger.error(f"处理持仓UI显示失败: {e}, 持仓数据: {position}")
                    import traceback
                    self.logger.error(f"错误堆栈: {traceback.format_exc()}")
                    continue

            # 检查表格行数
            table_row_count = self.position_table.row_count
            self.logger.info(f"持仓表格UI更新完成，app_core有 {len(self.app_core.position_data)} 只持仓，表格显示 {table_row_count} 行")

            # 强制刷新表格显示
            self.position_table.refresh()
            self.logger.debug("持仓表格已强制刷新显示")

        except Exception as e:
            self.logger.error(f"更新持仓表格失败: {e}")
            import traceback
            self.logger.error(f"详细错误: {traceback.format_exc()}")

    async def update_position_display(self) -> None:
        """更新持仓信息显示（兼容性方法，指向update_position_table）"""
        await self.update_position_table()

    async def add_stock_to_table(self, stock_code: str) -> None:
        """添加股票到表格"""

        if self.stock_table:
            self.stock_table.add_row(
                stock_code,
                stock_code,
                "0.00",
                "0.00%", 
                "0",
                "未更新",
                key=stock_code
            )
    
    async def remove_stock_from_table(self, stock_code: str) -> None:
        """从表格删除股票"""
        if self.stock_table:
            try:
                self.stock_table.remove_row(stock_code)
            except Exception as e:
                self.logger.warning(f"从表格删除股票行失败: {e}")
    
    async def update_status_bar(self) -> None:
        """更新状态栏各个组件的显示内容"""
        try:
            from datetime import datetime
            from base.monitor import ConnectionStatus, MarketStatus
            
            # 更新连接状态
            if self.connection_status:
                if self.app_core.connection_status == ConnectionStatus.CONNECTED:
                    self.connection_status.update("🟢 已连接")
                elif self.app_core.connection_status == ConnectionStatus.DISCONNECTED:
                    self.connection_status.update("🟡 未连接")
                else:
                    self.connection_status.update("🔴 连接错误")
            
            # 更新市场状态
            if self.market_status:
                if self.app_core.market_status == MarketStatus.OPEN and self.app_core.open_markets:
                    open_markets_text = ",".join(self.app_core.open_markets)
                    self.market_status.update(f"📈 开盘({open_markets_text})")
                elif self.app_core.market_status == MarketStatus.OPEN:
                    self.market_status.update("📈 开盘")
                else:
                    self.market_status.update("📉 闭市")
            
            # 更新刷新模式
            if self.refresh_mode:
                mode_text = getattr(self.app_core, 'refresh_mode', '未知模式')
                self.logger.info(f"正在更新刷新模式显示: {mode_text}")
                self.refresh_mode.update(f"🔄 {mode_text}")
                self.logger.info(f"刷新模式显示更新完成: 🔄 {mode_text}")
            else:
                self.logger.warning("刷新模式组件引用为空，无法更新显示")
            
            
            # 更新最后更新时间
            if self.last_update:
                current_time = datetime.now()
                time_str = current_time.strftime("%H:%M:%S")
                self.last_update.update(f"更新: {time_str}")
            
            self.logger.debug("状态栏更新完成")
            
        except Exception as e:
            self.logger.error(f"更新状态栏失败: {e}")
    
    async def update_cell_with_flash(self, stock_code: str, column: str, value: str, 
                                   change_rate: float = None) -> None:
        """
        更新表格单元格并应用0.5秒的颜色闪烁效果
        只有当值发生变化时才显示闪烁效果
        
        Args:
            stock_code: 股票代码
            column: 列名
            value: 更新的值
            change_rate: 涨跌幅(用于price和change列的颜色判断)
        """
        if not self.stock_table:
            return
            
        try:
            # 生成单元格的唯一键
            cell_key = f"{stock_code}:{column}"
            
            # 检查值是否发生变化
            last_value = self.last_cell_values.get(cell_key)
            has_changed = last_value != value
            
            # 更新缓存值
            self.last_cell_values[cell_key] = value
            
            if has_changed:
                # 值发生变化，应用闪烁效果
                self.logger.debug(f"数据变化检测: {cell_key} '{last_value}' -> '{value}'")
                
                # 根据列类型选择闪烁颜色
                if column in ['price', 'change']:
                    # 价格和涨跌相关列：使用黄色背景突出显示
                    flash_value = f"[bold yellow on blue]{value}[/bold yellow on blue]"
                else:
                    # 其他列：使用蓝色背景
                    flash_value = f"[bold white on blue]{value}[/bold white on blue]"
                
                # 立即更新为闪烁样式
                self.stock_table.update_cell(stock_code, column, flash_value)
                
                # 创建异步任务，0.5秒后恢复正常样式
                asyncio.create_task(
                    self._restore_cell_normal_style(stock_code, column, value, change_rate)
                )
            else:
                # 值未变化，直接更新为正常样式（不闪烁）
                self.logger.debug(f"数据无变化: {cell_key} 保持值 '{value}'")
                
                # 直接应用正常样式
                if column in ['price', 'change'] and change_rate is not None:
                    if change_rate > 0:
                        # 上涨：红色
                        normal_value = f"[bold red]{value}[/bold red]"
                    elif change_rate < 0:
                        # 下跌：绿色
                        normal_value = f"[bold green]{value}[/bold green]"
                    else:
                        # 平盘：默认颜色
                        normal_value = value
                else:
                    # 其他列使用默认颜色
                    normal_value = value
                
                self.stock_table.update_cell(stock_code, column, normal_value)
            
        except Exception as e:
            self.logger.error(f"应用单元格闪烁效果失败: {e}")
            # 失败时直接更新为正常值
            self.stock_table.update_cell(stock_code, column, value)
    
    async def _restore_cell_normal_style(self, stock_code: str, column: str, value: str, 
                                       change_rate: float = None) -> None:
        """
        0.5秒后恢复单元格的正常样式
        
        Args:
            stock_code: 股票代码
            column: 列名
            value: 原始值
            change_rate: 涨跌幅(用于确定颜色)
        """
        try:
            # 等待0.5秒
            await asyncio.sleep(0.5)
            
            # 根据列类型和涨跌情况应用正常样式
            if column in ['price', 'change'] and change_rate is not None:
                if change_rate > 0:
                    # 上涨：红色
                    normal_value = f"[bold red]{value}[/bold red]"
                elif change_rate < 0:
                    # 下跌：绿色
                    normal_value = f"[bold green]{value}[/bold green]"
                else:
                    # 平盘：默认颜色
                    normal_value = value
            else:
                # 其他列使用默认颜色
                normal_value = value
            
            # 恢复正常样式
            if self.stock_table:
                self.stock_table.update_cell(stock_code, column, normal_value)
                
        except Exception as e:
            self.logger.error(f"恢复单元格正常样式失败: {e}")
    
    async def notify_analysis_panel_created(self) -> None:
        """通知lifecycle管理器AnalysisPanel已创建"""
        try:
            self.logger.info("尝试通知lifecycle管理器AnalysisPanel已创建")
            
            # 尝试多种方式获取lifecycle_manager
            lifecycle_manager = None
            
            # 方式1: 从app_core直接获取
            if hasattr(self.app_core, 'lifecycle_manager'):
                lifecycle_manager = self.app_core.lifecycle_manager
                self.logger.debug("从app_core获取到lifecycle_manager")
            
            # 方式2: 从app获取
            elif hasattr(self.app_core, 'app') and hasattr(self.app_core.app, 'lifecycle_manager'):
                lifecycle_manager = self.app_core.app.lifecycle_manager
                self.logger.debug("从app_core.app获取到lifecycle_manager")
            
            self.logger.debug(f"最终获取到lifecycle_manager: {lifecycle_manager}")
            
            if lifecycle_manager and hasattr(lifecycle_manager, 'setup_analysis_panel_welcome'):
                lifecycle_manager.setup_analysis_panel_welcome()
                self.logger.info("已成功通知lifecycle管理器AnalysisPanel创建")
            else:
                self.logger.warning(f"lifecycle_manager未找到或没有setup_analysis_panel_welcome方法，manager={lifecycle_manager}")
        except Exception as e:
            self.logger.error(f"通知AnalysisPanel创建失败: {e}")
    
    # ================== 标签页管理方法 ==================
    
    async def create_analysis_tab(self, stock_code: str) -> bool:
        """
        创建分析标签页
        
        Args:
            stock_code: 股票代码 (如 HK.00700, US.AAPL)
            
        Returns:
            bool: 创建是否成功
        """
        try:
            self.logger.info(f"开始创建分析标签页: {stock_code}")
            
            # 获取主标签页容器
            main_tabs = self.app.query_one("#main_tabs", expect_type=None)
            if not main_tabs:
                self.logger.error("找不到主标签页容器 #main_tabs")
                return False
            
            # 生成标签页ID和标题
            tab_id = f"analysis_{stock_code.replace('.', '_')}"
            tab_title = f"📊 {stock_code}"
            
            # 检查标签页是否已存在
            existing_panes = list(main_tabs.query("TabPane"))
            for pane in existing_panes:
                if pane.id == tab_id:
                    self.logger.info(f"分析标签页 {tab_id} 已存在，激活该标签页")
                    #main_tabs.active = tab_id
                    return True
            
            # 导入分析界面组件
            try:
                from monitor.monitor_layout import AnalysisPanel
                from textual.widgets import TabPane
            except ImportError as e:
                self.logger.error(f"导入分析界面组件失败: {e}")
                return False
            
            # 创建新的分析面板
            analysis_content = AnalysisPanel(id="analysis_panel")
            analysis_content.set_app_reference(self.app)
            
            # 创建TabPane并添加到主标签页容器
            new_tab_pane = TabPane(tab_title, analysis_content, id=tab_id)
            await main_tabs.add_pane(new_tab_pane)
            
            # 激活新创建的标签页
            #main_tabs.active = tab_id
            
            # 等待一小段时间让标签页完全创建
            import asyncio
            await asyncio.sleep(0.1)
            
            # 加载股票分析数据
            analysis_data_manager = getattr(self.app_core, 'analysis_data_manager', None)
            if analysis_data_manager:
                # 异步设置当前股票并加载数据
                success = await analysis_data_manager.set_current_stock(stock_code)
                if success:
                    # 通知AnalysisPanel股票已切换
                    await analysis_content.on_stock_changed(stock_code)
                    self.logger.info(f"已为股票 {stock_code} 加载分析数据")
                else:
                    self.logger.error(f"加载股票 {stock_code} 分析数据失败")
            else:
                self.logger.error("AnalysisDataManager未初始化")
            
            # 通知分析面板已创建
            await self.notify_analysis_panel_created()
            
            self.logger.info(f"成功创建分析标签页: {tab_id}")
            return True
            
        except Exception as e:
            self.logger.error(f"创建分析标签页 {stock_code} 失败: {e}")
            return False
    
    async def close_analysis_tab(self, stock_code: str = None) -> bool:
        """
        关闭分析标签页
        
        Args:
            stock_code: 要关闭的股票代码。如果为None，则关闭当前激活的分析标签页
            
        Returns:
            bool: 关闭是否成功
        """
        try:
            self.logger.info(f"开始关闭分析标签页: {stock_code or '当前激活'}")
            
            # 获取主标签页容器
            main_tabs = self.app.query_one("#main_tabs", expect_type=None)
            if not main_tabs:
                self.logger.error("找不到主标签页容器 #main_tabs")
                return False
            
            tab_id_to_close = None
            
            if stock_code:
                # 关闭指定股票的分析标签页
                tab_id_to_close = f"analysis_{stock_code.replace('.', '_')}"
            else:
                # 关闭当前激活的分析标签页（如果是分析标签页）
                current_active = main_tabs.active
                if current_active and current_active.startswith("analysis_"):
                    tab_id_to_close = current_active
                else:
                    self.logger.warning("当前激活的标签页不是分析标签页，无法关闭")
                    return False
            
            # 检查标签页是否存在
            existing_panes = list(main_tabs.query("TabPane"))
            pane_exists = any(pane.id == tab_id_to_close for pane in existing_panes)
            
            if not pane_exists:
                self.logger.warning(f"分析标签页 {tab_id_to_close} 不存在")
                return False
            
            # 在关闭前停止相关的实时数据更新
            if stock_code:
                analysis_data_manager = getattr(self.app_core, 'analysis_data_manager', None)
                if analysis_data_manager:
                    try:
                        # 清理该股票的实时数据任务
                        await analysis_data_manager.cleanup_stock_data(stock_code)
                        self.logger.debug(f"已清理股票 {stock_code} 的分析数据")
                    except Exception as e:
                        self.logger.warning(f"清理分析数据失败: {e}")
            
            # 移除标签页
            await main_tabs.remove_pane(tab_id_to_close)
            
            # 如果关闭的是当前激活标签页，切换到主界面
            if main_tabs.active == "" or main_tabs.active == tab_id_to_close:
                main_tabs.active = "main"
            
            self.logger.info(f"成功关闭分析标签页: {tab_id_to_close}")
            return True
            
        except Exception as e:
            self.logger.error(f"关闭分析标签页失败: {e}")
            return False
    
    async def close_current_tab(self) -> bool:
        """
        关闭当前激活的标签页（仅限分析标签页）
        这个方法专门用于Cmd+W快捷键
        
        Returns:
            bool: 关闭是否成功
        """
        try:
            main_tabs = self.app.query_one("#main_tabs", expect_type=None)
            if not main_tabs:
                return False
                
            current_active = main_tabs.active
            
            # 只允许关闭分析标签页，保护主界面标签页
            if current_active and current_active.startswith("analysis_"):
                # 从tab_id中提取股票代码
                stock_code = current_active.replace("analysis_", "").replace("_", ".")
                return await self.close_analysis_tab(stock_code)
            else:
                self.logger.debug(f"当前标签页 {current_active} 不是分析标签页，无法关闭")
                return False
                
        except Exception as e:
            self.logger.error(f"关闭当前标签页失败: {e}")
            return False
    
    def has_analysis_tab(self, stock_code: str) -> bool:
        """检查分析标签页是否存在"""
        try:
            main_tabs = self.app.query_one("#main_tabs", expect_type=None)
            if not main_tabs:
                return False
            
            tab_id = f"analysis_{stock_code.replace('.', '_')}"
            existing_panes = list(main_tabs.query("TabPane"))
            
            for pane in existing_panes:
                if pane.id == tab_id:
                    return True
            
            return False
            
        except Exception as e:
            self.logger.error(f"检查分析标签页存在性失败: {e}")
            return False